<?php
include('meter-custom.inc.php');

define( 'FORMAT_FT1_1',         0x04 );
define( 'FORMAT_FT1_2',         0x10 );
define( 'FORMAT_FT1_2_LONG',    0x68 );
define( 'FORMAT_FT1_2_END',     0x16 );

define( 'CONTROL_MASK_BIT_PRM', 0x40);

define( 'CMD_READ_MASTER_TKON'        ,0x01 );
define( 'CMD_READ_MASTER_TKON_INDEX'  ,0x15 );

define( 'CMD_READ_SLAVE_TKON'         ,0x11 );


define( 'PARAM_TYPE',           0xF000 );
define( 'PARAM_SERIAL',         0xF001 );
define( 'PARAM_REZHIM',         0xF01A );
define( 'PARAM_F01D',           0xF01D );
define( 'PARAM_NAME',           0xF034 );
define( 'PARAM_QUERY',          0xF004 );
define( 'PARAM_QUERY_PARAM',    0xF00A );
define( 'PARAM_QUERY_PARAM_DESCR',0xF00B );


function L( $V )
{
  return ( $V & 0xff );
}

function H( $V )
{
  return $V >> 8;
}

class MeterTkon extends Meter
{
  public $can_num    = 0x01;
  public $can_num_by = 0x00;
  
  static function TestAllPacket( $Buffer, $Len )
  {
    if($Len < 3 )
      return false;

    if( 
      ord($Buffer[0])== FORMAT_FT1_2 
      and ord($Buffer[$Len-1])==FORMAT_FT1_2_END
    )
    {
      return true;
    }

    if( 
      ord($Buffer[0])== FORMAT_FT1_2_LONG 
      and ord($Buffer[$Len-1])==FORMAT_FT1_2_END
    )
    {
      return true;
    }


    return false;
  }
  
  function CulcKS( $arPacket, $ltrim, $rtrim )
  {
    $ks = 0;
    for( $i=$ltrim; $i<count($arPacket)-1-$rtrim; $i++ )
    {
      $ks = ($ks+$arPacket[$i])&0xff;
    }
    return $ks;
  }

  function UpdateCRC( &$arPacket )
  {
    if( $arPacket[0]== FORMAT_FT1_2_LONG )
    {
      $CRC = $this->CulcKS( $arPacket, 4, 1 );
    }
    else
    {
      $CRC = $this->CulcKS( $arPacket, 1, 1 );
    }
    $arPacket[ count($arPacket)-2 ] = $CRC;
  }

  function CheckPacket( $arPacket )
  {
    if( $arPacket[0]== FORMAT_FT1_2_LONG )
    {
      $CRC = $this->CulcKS( $arPacket, 4, 1 );
    }
    else
    {
      $CRC = $this->CulcKS( $arPacket, 1, 1 );
    }
    if( $arPacket[ count($arPacket)-2 ] != $CRC )
    {
      echo $arPacket[ count($arPacket)-2 ]." != ". $CRC;
      return 0;
    }
    /*
    if( $arPacket[6]>>4 == CLASS_ACCESS_ERROR )
      return -1;
    */
    $Size = $arPacket[1];
    if( $Size > 0 )
      return $Size;
    return 0;
  }

  function GenerateQueryPacket( $Bytes )
  {
    $packet_num = 0;
    $arPacket = Array(
      FORMAT_FT1_2,
      CONTROL_MASK_BIT_PRM+$packet_num,
      $this->can_num,
      CMD_READ_MASTER_TKON
    );
    foreach( $Bytes as $Byte ) 
      array_push( $arPacket, $Byte );
    array_push( $arPacket, CRC_INIT, FORMAT_FT1_2_END );
    $this->UpdateCRC( $arPacket );
    return $arPacket;
  }
  
  function GenerateQueryPacketIndex( $Bytes )
  {
    $PktLen = 8;
    $packet_num = 0;
    $arPacket = Array(
      FORMAT_FT1_2_LONG,
      $PktLen,
      $PktLen,
      FORMAT_FT1_2_LONG,
      CONTROL_MASK_BIT_PRM+$packet_num,
      $this->can_num,
      CMD_READ_MASTER_TKON_INDEX
    );
    foreach( $Bytes as $Byte ) 
      array_push( $arPacket, $Byte );
    array_push( $arPacket, CRC_INIT, FORMAT_FT1_2_END );
    $this->UpdateCRC( $arPacket );
    return $arPacket;
  }


  
  function GetSubType()
  {
    if( !$this->IsLink() )
      return false;
    $arPacket = $this->GenerateQueryPacket( Array( L(PARAM_TYPE), H(PARAM_TYPE), 0 ) );
    if( $arPacket = $this->Converter->Send( $arPacket, Array( 'MeterTkon', 'TestAllPacket') ) )
    {
    //var_dump( PacketDumpReturn($arPacket ) );
      $Size = $this->CheckPacket( $arPacket ); 
      if( $Size == 4 )
      {
        return ($arPacket[7]<<8) + $arPacket[6];
      }
      return false;
    }
    else
      return false;
  }
  
  function GetSerialNumber()
  {
    if( !$this->IsLink() )
      return false;
    $arPacket = $this->GenerateQueryPacket( Array( L(PARAM_SERIAL), H(PARAM_SERIAL), 0 ) );
    if( $arPacket = $this->Converter->Send( $arPacket, Array( 'MeterTkon', 'TestAllPacket') ) )
    {
    //var_dump( PacketDumpReturn($arPacket ) );
      $Size = $this->CheckPacket( $arPacket ); 
      if( $Size == 4 )
      {
        return ($arPacket[7]<<8) + $arPacket[6];
      }
      return false;
    }
    else
      return false;
  }
  
  function GetDatetime()
  {
    if( !$this->IsLink() )
      return false;
    // SERV_DIRECT_QUERY | SERV_ACCESS_CMD | 0U;
    $arPacket = $this->GenerateQueryPacket( Array( 0x17, 0xF0, 0x00 ) );
    $arPacketDate = $this->Converter->Send( $arPacket, Array('MeterTkon', 'TestAllPacket') );
    if( $arPacketDate )
    {
      $arPacket = $this->GenerateQueryPacket( Array( 0x18, 0xF0, 0x00 ) );
      $arPacketTime = $this->Converter->Send( $arPacket, Array('MeterTkon', 'TestAllPacket') );
      if( $arPacketTime )
      {
        //var_dump( $arPacket );
        $date = new DateTime();
        $date->setDate( 
          2000+BCDfrom( $arPacketDate[3+3] ), // Год
          BCDfrom( $arPacketDate[3+2] ),      // Месяц
          BCDfrom( $arPacketDate[3+1] )       // День
        );
        $date->setTime( 
          BCDfrom( $arPacketTime[3+3] ), // Часы
          BCDfrom( $arPacketTime[3+2] ), // Минуты
          BCDfrom( $arPacketTime[3+1] )  // Секунды
        );
        
  //      $date = date_create('2001-01-01');
        echo date_format( $date, 'Y-m-d H:i:s' ) . "\n";
        return $date;
      }
    }
    return false;
  }
  
  function GetQueryInfo()
  {
    if( !$this->IsLink() )
      return false;
    $Count = 0x20;
    echo "<table>";
    echo "<tr><td>NN</td><td>Алгоритм</td><td>Параметры</td></tr>";
    for( $Start = 0; $Start< 20; $Start += $Count )
    {
      $arPacket = $this->GenerateQueryPacketIndex( Array( 0x29, 0xF0, L($Start), H($Start), $Count ) );
      if( $arPacket = $this->Converter->Send( $arPacket, Array( 'MeterTkon', 'TestAllPacket') ) )
      {
        $Size = $this->CheckPacket( $arPacket ); 
        $arPacket = array_slice( $arPacket, 6, $Size-2 );
        if( $Size == 2+$Count )
        {
          for( $i=0; $i<$Count; $i++)
          {
            /*
            if( $arPacket[$i+1]==0xFF and $arPacket[$i+0]==0xFF )
            {
              echo "</table>";
              return true;
            }*/
            echo "<tr><td>".($Start+$i+1).": </td><td>"
              .sprintf( "0x%02X %s", $arPacket[$i], mb_convert_encoding( chr($arPacket[$i]), 'UTF-8', 'Windows-1251' ) )
              ."</td></tr>";
          }
  //        return ($arPacket[7]<<8) + $arPacket[6];
        }
        else
          return false;
      }
      else
        return false;
    }
    echo "</table>";
echo "<hr />";


    echo "<table>";
    echo "<tr><td>NN</td><td>Алгоритм</td><td>Параметры</td></tr>";
    for( $Start = 0; $Start< 20; $Start += $Count )
    {
      $arPacket = $this->GenerateQueryPacketIndex( Array( 0x04, 0xF0, L($Start), H($Start), $Count ) );
      if( $arPacket = $this->Converter->Send( $arPacket, Array( 'MeterTkon', 'TestAllPacket') ) )
      {
        $Size = $this->CheckPacket( $arPacket ); 
        $arPacket = array_slice( $arPacket, 6, $Size-2 );
        if( $Size == 2+4*$Count )
        {
          for( $i=0; $i<$Count; $i++)
          {
            if( $arPacket[$i*4+1]==0xFF and $arPacket[$i*4+0]==0xFF )
            {
              echo "</table>";
              return true;
            }
            echo "<tr><td>".($Start+$i+1).": </td><td>"
              .sprintf( "0x%02X%02X", $arPacket[$i*4+1], $arPacket[$i*4+0] )
              ."</td><td>"
              .sprintf( "0x%02X%02X", $arPacket[$i*4+3], $arPacket[$i*4+2] )
              ."</td></tr>";
          }
  //        return ($arPacket[7]<<8) + $arPacket[6];
        }
        else
          return false;
      }
      else
        return false;
    }
    echo "</table>";

    //return true;
  

echo "<hr />";
echo "<h3>Таблица параметров загружаемых задач i=0..2047</h3>";
    echo "<table>";
    echo "<tr><td>NN</td><td>Алгоритм</td><td>Параметры</td></tr>";
    for( $Start = 0; $Start< 20; $Start += $Count )
    {
      $arPacket = $this->GenerateQueryPacketIndex( Array( 0x0A, 0xF0, L($Start), H($Start), $Count ) );
      if( $arPacket = $this->Converter->Send( $arPacket, Array( 'MeterTkon', 'TestAllPacket') ) )
      {
        $Size = $this->CheckPacket( $arPacket ); 
        $arPacket = array_slice( $arPacket, 6, $Size-2 );
        if( $Size == 2+4*$Count )
        {
          for( $i=0; $i<$Count; $i++)
          {
            if( $arPacket[$i*4+1]==0xFF and $arPacket[$i*4+0]==0xFF )
            {
              echo "</table>";
              return true;
            }
            echo "<tr><td>".($Start+$i+1).": </td><td>"
              .sprintf( "0x%02X%02X", $arPacket[$i*4+1], $arPacket[$i*4+0] )
              ."</td><td>"
              .sprintf( "0x%02X%02X", $arPacket[$i*4+3], $arPacket[$i*4+2] )
              ."</td></tr>";
          }
  //        return ($arPacket[7]<<8) + $arPacket[6];
        }
        else
          return false;
      }
      else
        return false;
    }
    echo "</table>";


echo "<hr />";
echo "<h3>Таблица описания гибких параметров i=0..1019</h3>";
$Count = 8;
    echo "<table>";
    echo "<tr><td>NN</td><td>Алгоритм</td><td>Параметры</td></tr>";
    for( $Start = 0; $Start< 20; $Start += $Count )
    {
      $arPacket = $this->GenerateQueryPacketIndex( Array( 0x0B, 0xF0, L($Start), H($Start), $Count ) );
      if( $arPacket = $this->Converter->Send( $arPacket, Array( 'MeterTkon', 'TestAllPacket') ) )
      {
        $Size = $this->CheckPacket( $arPacket ); 
        $arPacket = array_slice( $arPacket, 6, $Size-2 );
        if( $Size == 2+4*$Count )
        {
          for( $i=0; $i<$Count; $i++)
          {
            if( $arPacket[$i*4+1]==0xFF and $arPacket[$i*4+0]==0xFF )
            {
              echo "</table>";
              return true;
            }
            echo "<tr><td>".($Start+$i+1).": </td><td>"
              .sprintf( "0x%02X%02X", $arPacket[$i*4+1], $arPacket[$i*4+0] )
              ."</td><td>"
              .sprintf( "0x%02X%02X", $arPacket[$i*4+3], $arPacket[$i*4+2] )
              ."</td></tr>";
          }
  //        return ($arPacket[7]<<8) + $arPacket[6];
        }
        else
          return false;
      }
      else
        return false;
    }
    echo "</table>";





    return true;
  }
 
  
}

