<?php
/*
Имя класса place
Имя файла place.class.php ( placecustom.class.php )
Поля класса:
Id : id
Title : string
Place : object
Meters : multy
Sort : int
Interprise : object
Lft : int
Rgt : int
Map : string
Isgraphic : bool
Placetype : object
*/
class place extends placecustom{
  
  
  
  /*
    Таблица соответствия для работы с JSON
  */
  public $FieldsToJSON = array(
    'Id'                => 'id',
    'Title'             => 'title',
    'Place'             => 'place',
    'Meters'            => 'meters',
    'Lft'               => 'lft',
    'Rgt'               => 'rgt',
    'Map'               => 'map',
    'Isgraphic'         => 'is_graphic',
    'Placetype'         => 'type',
    'BankBook'          => 'bankbook',
  );  
  
  
  
  
  
  /**
   * Получение строки хлебных крошек в текстовом варианте
   * 
   * @return string
   */
  function GetBreadcrumbsParentToThisReports(){
    $str = "";
    foreach( $this->GetParentAll()->Elements as $iPlace ){
      $str .= $iPlace->Title ." / ";
      $str .= $iPlace->BankBook;
    }
    return $str;
  }
  
  
  
  
  
  /**
   * Получение группировки данных по параметру и диапазону дат
   * 
   * @param parameter $Parameter
   * @param string $date_start
   * @param string $date_stop
   * @return meterdatahour
   */
  function GetMDHGroupByParameterRangeDate( $Parameter , $date_start , $date_stop ){
    
    $start  = new DateTime( $date_start );
    $stop   = new DateTime( $date_stop  );
    
    DB::Query("Drop table if exists mdh_by_meter");
    
    DB::Query(Utils::Strip("
      CREATE TEMPORARY TABLE IF NOT EXISTS mdh_by_meter (
        year int(11) NOT NULL, 
        month int(11), 
        day int(11), 
        meter_id int(11),
        parameter_id int(11)
      )            
    "));
    
    $value = false;
    
    $SQL_A = array();
    
    $limit_trans = 300; // Максимальное кол-во вставок в строке
    
    foreach( $this->GetMetersAll()->Elements as $iMeter ){

      while( $start->getTimestamp() <= $stop->getTimestamp() ){

        $SQL_A[] = "({$start->format('Y,m,d')},{$iMeter->Id},{$Parameter->Id})";

        if( count( $SQL_A ) == $limit_trans ){
          $SQL = "INSERT INTO mdh_by_meter (`year`,`month`,`day`,`meter_id`,`parameter_id`) values" . implode( "," , $SQL_A ) . "";
          DB::Query($SQL);
          $SQL_A = array();
        }

        $start->modify( "+1 day" );
      }
    }
    
    if( $SQL_A ){
      $SQL = "INSERT INTO mdh_by_meter (`year`,`month`,`day`,`meter_id`,`parameter_id`) values" . implode( "," , $SQL_A ) . "";
      DB::Query($SQL);
    }
    
    $res = DB::Fetch(DB::Select(Utils::Strip("
      select 
      1 as id, 
      mdh.parameter_id, 
      m.tarif_id, 
      mdh.year, 
      mdh.month, 
      mdh.day, 
      m.hour, 
      m.loginfo_id, 
      m.add_time, 
      SUM(m.value) as value
      from mdh_by_meter as mdh 
      left join meterdatahour as m on 
      mdh.year = m.year and 
      mdh.month = m.month and 
      mdh.day = m.day and 
      mdh.meter_id = m.meter_id and 
      mdh.parameter_id = m.parameter_id 
    ")));

    if( $res ){

      return meterdatahour::SetFieldsFromObject( $res );
    }
    
    return meterdatahour::Id(0);
  }
  
  
  
  
  
  
  /**
   * Действия после добавления места в БД
   * 
   * 1. Проверка и вставка лицевого счета
   * 
   * @param array $A Массив со значениями, с которыми добавляли место
   * @return boolean
   */
  function OnAfterSqlInsert( $A ){
    if( $this->IssetBankBookRequest( $A ) ){
      $this->SetBankBook( $this->GetBankBookRequest( $A ) );
    }
    return true;
  }
  
  
  
  
  /**
   * Действия после добавления места в БД
   * 
   * 1. Проверка и вставка лицевого счета
   * 
   * @param array $A Массив со значениями, с которыми добавляли место
   * @return boolean
   */
  function OnAfterSqlUpdate( $A ){
    if( $this->IssetBankBookRequest( $A ) ){
      $this->SetBankBook( $this->GetBankBookRequest( $A ) );
    }
    return true;
  }
  
  
  
  
  
  
  
  
  
  /**
   * Получение названия места
   * 
   * @return string
   */
  function GetTitle(){
    
    if( !$this->_F['Title']['ready'] ){
      TObject2::_DataLoadSql($this);
    }
    
    if( !$this->_F['Title']['value'] ){
      $this->Title = $this->Interprise->Title;
    }
    
    return $this->_F['Title']['value'];
  }
  
  
  
  
  /*
    Флаг логирования
  */
  public $NeedDoUserLog = true;
  
  
  
  
  
  /**
   * Генерация названия эекпорта файла
   * 
   * @return string
   */
  function GetNameReport(){
    
    $name = $this->Place->Id ? Utils::str2url($this->Title) : Utils::str2url($this->Interprise->Title);
    
    $y = ceil( $this->GetDate()->format("Y") );
    $m = ceil( $this->GetDate()->format("m") );
    $d = ceil( $this->GetDate()->format("d") );
    
    switch( $this->GetStepArchive() ){
      case "day"    : $m++; break;
      case "month"    : $y++; break;
    }
    
    if( ceil($m) < 10 ) $m = "0{$m}";
    if( ceil($d) < 10 ) $d = "0{$d}";
    
    switch( $this->GetStepArchive() ){
      case "day"    : return "{$name}_{$this->Id}_{$y}_{$m}";
      case "month"  : return "{$name}_{$this->Id}_{$y}";
    }

    $prefixId = $this->Place->Id ? "_{$this->Id}" : "";

    if( isset($_GET['change_filter_ht']) ){
      if( $_GET['change_filter_ht'] == "HeatCounter" ){
        $prefixId .= "_heat";
      }elseif( $_GET['change_filter_ht'] == "AquaCounter" ){
        $prefixId .= "_water";
      }else{
        $prefixId .= "_electical";
      }
    }
    
    return "{$name}{$prefixId}_{$y}_{$m}_{$d}";
  }
  
  
  
  /*
    Игнорируемые поля для занесения в лог
  */
  public $IgnoreDoUserLog = array(
    'Rgt',
    'Lft'
  );
  
  
  
  /*
    Хранит в сеье кеш кол-ва приборов учета
  */
  public $ChildrenReadMetersByStatusesCache = null;

  /*
  
    ВНИМАНИЕ. для получения значения на теущее место
    нужно обращатся через родителя
    
    @param place Место на которое хотим узнать колв- приборв учета
    @return Array
  
    Получает у всех дочерних элементов приборы учета и закидывает их 
    в _OtherData
        meter_ok_count
        meter_notok_count
        meter_excluded_count
  */
  function ChildrenReadMetersByStatuses( $oPlace ){
    
    if( $this->ChildrenReadMetersByStatusesCache === null ){
    
      $SQL = Utils::Strip("
        select 
          p.place_id,
          p.lft,
          p.rgt,
          count( p.error ) as error,
          count( p.excluded ) as excluded,
          count( p.id ) as summ
        from (
          select 
            meterplace.*,
            place.lft,
            place.rgt,
            meter_error.id as error,
            meter_excluded.id as excluded
          from place
          left outer join meterplace
            on meterplace.place_id = place.id
          left join meter
            on meter.id = meterplace.meter_id
          left outer join meter as meter_error
            on meter.id = meter_error.id and meter_error.is_ok = 0 
          left outer join meter as meter_excluded
            on meter.id = meter_excluded.id and meter_excluded.is_excluded = 1
          left outer join meter as meter_ok
            on meter.id = meter_ok.id and meter_ok.is_ok = 1
          where place.lft >= {$this->Lft} and
          place.interprise = ".Self()->Interprise->Id."  and
          meter.interprise = ".Self()->Interprise->Id."  and
          meter.converter_id > 0  and
          meterplace.in_path = 0 and 
          meterplace.place_id = place.id
          group by meterplace.place_id,meterplace.meter_id
        ) as p
        group by p.place_id
      ");
      
      $res = DB::Select( $SQL );
        
      while( $row = DB::Fetch( $res ) ){
        $this->ChildrenReadMetersByStatusesCache[ $row['place_id'] ] = $row;
      }
    }
    
    $error    = 0;
    $all      = 0;
    $excluded = 0;
    
    if( $this->ChildrenReadMetersByStatusesCache !== null ){
      foreach( $this->ChildrenReadMetersByStatusesCache as $settings ){
        if( $settings['lft'] >= $oPlace->Lft && $settings['rgt'] <= $oPlace->Rgt ){
          $error = $error + $settings['error'];
          $all = $all + $settings['summ'];
          $excluded = $excluded + $settings['excluded'];
        }
      }
    }
    
    return array(
      'all'       => $all,
      'error'     => $error,
      'excluded'  => $excluded
    );
    
  }
  
  
  
  
  
  
  
  
  /*
    Переключатель который говорит о том создавалась таблица или нет
  */
  protected $IsTemp_place_mdh = false;
  
  
  /*
    Создание временной таблицы чтобы попадать в ключ при выборке
    из данных
    @return VOID
  */
  protected function Temp_place_mdh(){
    
    
    /*
      На сдучай если таблица не создана
    */
    $Filter = array( 
      'Limit' => 1
    );
    meterparameter::Filter($Filter)->First->Id;
    $Filter = array( 
      'Limit' => 1
    );
    meterplace::Filter($Filter)->First->Id;
    
    /*
      Создаем временную таблицу с приборами учета которые нам неоьбходимо посчитать
    */
    
    $SQL = Utils::Strip( "drop table place_mdh" );
    DB::Select($SQL);
    
    $Y = $this->Date->format('Y');
    $M = $this->Date->format('m');
    $D = $this->Date->format('d');
    
    if( !isset( $_GET['day'] ) && isset( $_GET['month'] ) ){
      $D = 0;
      $M++;
    }
    if( !isset( $_GET['month'] ) && isset( $_GET['year'] ) ){
      $D = 0;
      $M = 0;
      $Y++;
    }
    
    
    $Place = $this;
    
//    if( $this->Id && ( !$this->Lft || !$this->Rgt ) ){
//      self::globalReplacePointer( Self()->Interprise->Id );
//      TObject2::CleanAllObjects();
//      $Place = place::Id( $this->Id );
//    }
    
    
    $meterplace_f = isset( meterplace::$FInit['InPath'] ) ? 'and meterplace.in_path = 0' : '' ;
    
    $Rgt = $this->Id
              ? "and place.rgt <= {$this->Rgt} "
              : "";
              
              
    // Если есть дополнительные типы то нужно сджойнить таблицы
    $Join_extra = "";
    $Where_extra = "";

    if( isset( $_GET['change_filter_ht'] ) ){

      // проверим валидность поля
      $Filter = array(
          'Filter' => array(
              'Ident' => $_GET['change_filter_ht']
          )
      );
      $ResExtra = meter_extra_type::Filter( $Filter );
      if( $ResExtra->Count ){
        $Join_extra = "left join meter_type ON meter.type_id = meter_type.id 
        left join meter_extra_type on meter_extra_type.id = meter.meter_extra_type_id
          or meter_type.meter_extra_type_id = meter_extra_type.id";
        $Where_extra = " and meter_extra_type.ident = '{$ResExtra->First->Ident}' ";
      }
    }
    
    $SQL = Utils::Strip( "
      CREATE TEMPORARY TABLE IF NOT EXISTS place_mdh 
      (
        year INTEGER NOT NULL , 
        month INTEGER NOT NULL , 
        day INTEGER NOT NULL , 
        meter_id INTEGER NOT NULL , 
        INDEX my_index_name (meter_id)) AS (
        select 
          {$Y} as year,
          {$M} as month,
          {$D} as day,
          meterplace.meter_id 
      from meterplace 
      left join place 
        on meterplace.place_id = place.id 
      left join meter
        on meter.id = meterplace.meter_id
      {$Join_extra}
      where place.interprise = ".Self()->Interprise->Id." and 
      place.lft >= {$this->Lft} {$Rgt}
      {$meterplace_f}
      {$Where_extra}
      Group by meterplace.meter_id 
      )
    " );
    
    //~ echo $SQL;
    
    // if( Self()->Interprise->Id == 190 ){
    //   echo $SQL;die();
    // }
    $this->IsTemp_place_mdh = ( bool ) DB::Select($SQL);
  }
  
  
  
  /**
   * 
   * Получение ресурса на выборку суммированных данных
   * по переданных параметрам и ВСЕМИ приборами на уровне и ниже этого места
   * 
   * @param array $parameters
   * @return boolean|\myqli
   */
  protected function GetSourceSummParameterByHours( $parameters ){
    
    
    if( !$this->IsTemp_place_mdh ){
      $this->Temp_place_mdh();
    }
    
    
    /*
      Создаем болванку таблицы и соединяем с place_mdh
      Должен получится результат 
        parameter_id | meter_id | hour | day | month | year
            1             1        ...
            2             1
            
      И соединяем с таблицей данных
      
      
      TODO подумать как переделать таблицу из памяти на шаблон
    */
    
    $arrayHours = array();
    for( $i = 1 ; $i <= 24 ; $i++ ){
      if( $i == 1 ){
        $arrayHours[] = "select * from (select {$i} as hour) as hour1";
        continue;
      }
      $arrayHours[] = "SELECT {$i} AS hour";
    }
    
    $Y = "place_mdh.year";
    $M = "place_mdh.month";
    $D = "place_mdh.day";
    
    if( isset( $_GET['year'] ) && isset( $_GET['month'] ) && isset( $_GET['day'] ) ){
      $Y = (int) $_GET['year'];
      $M = (int) $_GET['month'];
      $D = (int) $_GET['day'];
    }elseif( isset( $_GET['year'] ) && isset( $_GET['month'] ) && !isset( $_GET['day'] ) ){
      $Y = (int) $_GET['year'];
      $M = (int) $_GET['month'];
      $D = 0;
    }elseif( isset( $_GET['year'] ) && !isset( $_GET['month'] ) && !isset( $_GET['day'] ) ){
      $Y = (int) $_GET['year'];
      $M = 0;
      $D = 0;
    }
    
    
    
    $Filter = array( 
      'Filter' => array( 
        'Ident' => $parameters,
        'Enterprise' => 0
      ),
      'GroupBy' => array('Ident')
    );
    $arrayParameters = array();
    foreach( parameter::Filter($Filter)->Elements as $i => $iParameter ){
      if( $i == 0 ){
        $arrayParameters[] = "select * from (select {$iParameter->Id} as pid) as q1";
        continue;
      }
      $arrayParameters[] = "SELECT {$iParameter->Id} AS pid";
    }
    
    $SQL = Utils::Strip( "
      select 
        meterdatahour.*, 
        SUM(meterdatahour.value) as summ 
      from (
        ". implode(' union ' , $arrayParameters ) ."
      ) as ppid 
      left join 
      (
          ". implode(' union ' , $arrayHours ) ."
      ) as hours on 1=1
      left join place_mdh on 1=1
      left join meterdatahour
        on meterdatahour.meter_id = place_mdh.meter_id
        and meterdatahour.parameter_id = ppid.pid
        and meterdatahour.year = {$Y}
        and meterdatahour.month = {$M}
        and meterdatahour.day = {$D}
        and meterdatahour.hour = hours.hour
      WHERE meterdatahour.id IS NOT NULL
        GROUP BY /* meterdatahour.meter_id,*/ meterdatahour.parameter_id , meterdatahour.hour
    ");
    
    return DB::Select($SQL);
    
  }
  
  
  
  
  
  /**
   * Значение по умолчанию для данных на графике если нет данных
   * 
   * @var string 
   */
  protected $value_null_data_chart = "false";
  
  
  
  
  
  /**
   * Суммирование данных по всем РЕАЛЬНЫХ приборам учета по всему месту ниже По параметру ENERGY_TARIF1
   * 
   * @return string
   */
  function GetSumByEnergy1AllDown(){
    
    $ident = 'ENERGY_TARIF1';
    
    $parameter_idents = array( $ident );
    
    $resEnergy = parameter::GetByIdents( $parameter_idents );
    
    $r = $this->GetSumByParameterAllDownData( $parameter_idents );
    
    return "{
        lineWidth : 2,
        lineColor:'#7a92a3',
        name: '{$resEnergy->First->Lvl4}',
        data: [
          ".implode(',',$r)."
        ]
    }";
    
  }
  
  
  
  
  
  /**
   * Суммирование данных по всем РЕАЛЬНЫХ приборам учета по всему месту ниже По параметру ENERGY_TARIF2
   * 
   * @return string
   */
  function GetSumByEnergy2AllDown(){
    
    $ident = 'ENERGY_TARIF2';
    
    $parameter_idents = array( $ident );
    
    $resEnergy = parameter::GetByIdents( $parameter_idents );
    
    $r = $this->GetSumByParameterAllDownData( $parameter_idents );
    
    return "{
        lineWidth : 2,
        lineColor:'#0b62a4',
        name: '{$resEnergy->First->Lvl4}',
        data: [
          ".implode(',',$r)."
        ]
    }";
  }
  
  
  
  
  
  
  /**
   * Получение данных по всем сложенным приборам на этом и дочерних местах 
   * по идентифкаторам параметров
   * 
   * @param array $parameter_idents
   * @return array
   */
  function GetSumByParameterAllDownData( $parameter_idents ){
    
    $resEnergy = parameter::GetByIdents( $parameter_idents );

    if( !$resEnergy->Count ) return $this->value_null_data_chart;
    
    $Res = $this->GetSourceSummParameterByHours( $parameter_idents );
    
    $r = array();
    
    while( $row = DB::Fetch($Res) ){
      $r[ $row['parameter_id'] ][ $row['hour'] ] = meterdatahour::GetValueConvByParameter( $resEnergy->First , $row['summ'] );
    }
    
    $res = array();
    
    foreach( $r as $pid => $values ){
      for( $i = 1 ; $i <= 24 ; $i++ ){
        $res[] = !isset( $values[ $i ] ) ? 'false' : $values[ $i ];
      }
    }
    
    return array_values( $res );
  }
  
  
  
  
  
  
  /**
   * Получение ссумированных данных по 2 тарифам
   * 
   * @return array
   */
  function GetSumByEnergy1and2AllDownData(){
    $Data_ENERGY1 = $this->GetSumByParameterAllDownData( array( 'ENERGY_TARIF1' ) );
    $Data_ENERGY2 = $this->GetSumByParameterAllDownData( array( 'ENERGY_TARIF2' ) );
    
    
    $def = $this->value_null_data_chart;
    
    $r = array();
    
    if( $Data_ENERGY1 != $def && $Data_ENERGY2 == $def ){
      $r = $Data_ENERGY1;
    }elseif( $Data_ENERGY1 == $def && $Data_ENERGY2 != $def ){
      $r = $Data_ENERGY2;
    }else{
      foreach( $Data_ENERGY1 as $i => $v ){
        if( $Data_ENERGY2[ $i ] == $def && $Data_ENERGY1[ $i ] == $def ){
          $r[ $i ] = "false";
        }elseif( $Data_ENERGY1[ $i ] == $def && $Data_ENERGY2[ $i ] != $def ){
          $r[ $i ] = $Data_ENERGY2[ $i ];
        }elseif( $Data_ENERGY2[ $i ] == $def && $Data_ENERGY1[ $i ] != $def ){
          $r[ $i ] = $Data_ENERGY1[ $i ];
        }else{
          $r[ $i ] = $Data_ENERGY1[ $i ] + $Data_ENERGY2[ $i ];
        }
      }
    }
    return $r;
  }
  
  
  
  
  
  
  
  
  function GetSummByEnergyConversion(){
    return parameter::GetByIdents( array('ENERGY_TARIF1') )->First->Conversion;
  }
  
  function GetSummByEnergyRound(){
    return parameter::GetByIdents( array('ENERGY_TARIF1') )->First->Conversion->Round;
  }
  
  
  
  
  
  
  
  
  
  /**
   * Суммирование данных по всем РЕАЛЬНЫХ приборам учета по всему месту ниже
   * По параметру ENERGY_TARIF2 && ENERGY_TARIF1
   * 
   * @return string
   */
  function GetSumByEnergy1and2AllDown(){
    
    $r = $this->GetSumByEnergy1and2AllDownData();
    
    return "{
        lineWidth : 2,
        lineColor:'#4da74d',
        name: '".smarty()->get_config_vars('total_consumption').implode(',',$idents)."',
        data: [
          ".implode(',',$r)."
        ]
    }";
    
  }
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  /*
    Получение объекта фильтра параметров
    @return filterparamname
  */
  function GetFilterParameter( $sufix = "" ){
    $SQL = Utils::Strip("
      select filterparamname.* 
      from filterparamname 
      left join place 
        on place.id = filterparamname.object_id 
      where place.lft <= {$this->Lft} 
        and place.rgt >= {$this->Rgt} 
        and filterparamname.interprise_id = " . Self()->Interprise->Id . "
        and place.interprise = " . Self()->Interprise->Id . "
        order by place.lft desc
    ");
    $t = filterparamname::Id(0);
    $F = TListGroup::Prepare($SQL,$t);
    if( $F->Count ){
      return $F->First;
    }
    return filterparamname::Id(0);
  }
  
  
  
  
  
  
  /*
    Получение ссылок на этот плейс
    @return TListGroup
  */
  function GetUserLinks(){
    
    $Filter = array( 
      'Filter' => array( 
        'Interprise' => Self()->Interprise->Id,
        'Whyid' => $this->Id,
        'Auto' => 0,
        'Objectrule' => $this->Objectrule->Id,
      ),
    );
    return link::Filter( $Filter );
    
  }
  
  
  
  
  
  
  
  
  
  
  
  
  /*
    Преобразование объекта в массив для построение "дерева"
    @return Array
  */
  function GetToJsTree(){
    
    $parent = $this->Place->Is
              ? "place_tree_".$this->Place->Place->Id."_".$this->Place->Id
              : "#";
    
    return array(
      'text' => htmlspecialchars($this->Title,ENT_QUOTES),
      'id' => "place_tree_".$this->Place->Id."_".$this->Id,
      'children' => (bool)$this->Places->Count,
      'icon' => Site()->Config('icon.place'),
      'parent' => $parent
    );
  }
  
  
  
  
  
  
  
  
  
  /*
    Моржно ли добавлять в текущий элемент дочерние элементы
    @return Bool
  */
  function GetIsAddChildren(){
    return !$this->IsBank;
  }
  
  
  
  
  /*
    Название одной сущности для плейса исходя из его типа
    @return String
  */
  function GetConfigOneTitle(){
    return Site()->Config('ru.place');
  }
  
  
  
  
  
  
  
  
  /**
   * Проверка был ли передан банковский счет
   * 
   * @param array $A аргумент из TObject::StaticAddByArray TObject::StaticUpdateByArray
   * @return boolean
   */
  function IssetBankBookRequest( $A ){
    return isset( $A['__invalid'] ) && isset( $A['__invalid']['BankBook'] );
  }
  
  
  
  
  
  
  
  /**
   * Получение банковского счета из переданных аргументов
   * 
   * @param array $A аргумент из TObject::StaticAddByArray TObject::StaticUpdateByArray
   * @return boolean
   */
  function GetBankBookRequest( $A ){
    if( $this->IssetBankBookRequest( $A ) ){
      return $A['__invalid']['BankBook'];
    }
  }
  
  
  
  
  
  /**
   * Получение номера лицевого счета из БД
   * 
   * @return string
   */
  function GetBankBook(){
    return $this->GetBankBookField()->Value;
  }
  
  
  
  
  
  /**
   * Кеш значения дополнительного поля у места. Лицевой счет.
   * Нужно чтобы при добавлении нового, он не дублировался в БД
   *
   * @var fieldsvalue | null 
   */
  protected $CacheBankBook = null;
  
  
  
  
  
  /**
   * Получение экземпляра объекта доп. поля банковского счета
   * 
   * @return fieldsvalue
   */
  function GetBankBookField(){
    
    if( $this->CacheBankBook !== null ){
      return $this->CacheBankBook;
    }
    
    foreach( fieldsobject::GetFieldsForObject($this)->Elements as $iFields ){
      if( $iFields->Ident === 'BankBook' ){
        
        $Filter = array(
          'Filter' => array(
            'Fields'    => $iFields->Id,
            'Object'    => get_class( $this ),
            'Objectid'  => $this->Id
          )
        );
        $Res = fieldsvalue::Filter($Filter);
        
        if( !$Res->Count ){
          $this->CacheBankBook = fieldsvalue::AddFromObjectByIdent( $this , 'BankBook' );
          return $this->CacheBankBook;
        }
        $this->CacheBankBook = $Res->First;
        return $this->CacheBankBook;
      }
    }
    
    return fieldsvalue::Id(0);
  }














  /**
   * Установка номера лицевого счета месту
   * 
   * @param string $value
   * @return boolean
   */
  function SetBankBook( $value ){
    if( $this->GetIsBank() ){
      $BB = $this->GetBankBookField();
      $BB->Value = $value;
      $BB->Save();
    }
    return true;
  }
  
  
  
  
  
  
  
  /**
   * Является ли тип личным кабинетом
   * 
   * @return boolean
   */
  function GetIsBank(){
    return $this->Placetype->IsBank;
  }
  
  
  
  
  
  
  /*
    Является ли тип калькулятором
    @return Bool
  */
  function GetIsCalc(){
    return $this->Placetype->IsCalc;
  }
  
  
  
  
  /*
    Формат вывода данных для пользователя
  */
  protected $DateFormat = "d-m-Y";
  
  
  /*
    Формат вывода даты для ссылки
  */
  protected $DateFormatLink = "Y/m/d/";
  
  
  /*
    Кеш всех параметров
  */
  protected $CacheAllParameters = null;
  
  

  /*
    Получение непосредственных дочерних элементов данного места
    @return TListGroup
  */
  function GetChildren(){
    $placeLft = array();
    $placeRgt = array();


    $Filter = array(
      'Filter' => array(
        'Place' => $this->Id,
        'Interprise' => Self()->Interprise->Id,
        'Lft >=' => $placeLft,
        'Rgt <=' => $placeRgt
      )
    );
    if( Self()->GetIsUser() ){
      $RulePlaceFilter = array(
        'Filter' => array(
          'User' => Self()->Id
        )
      );
      $RulePlaceResFilter = ruleplace::Filter( $RulePlaceFilter );
      if( $RulePlaceResFilter->Count ){
        foreach ($RulePlaceResFilter->Elements as $iRulePlace ) {
          $placeLft[] = $iRulePlace->Place->Lft;
          $placeRgt[] = $iRulePlace->Place->Rgt;
        }
      }

      $Filter['Filter']['Lft >='] = $placeLft;
      $Filter['Filter']['Rgt <='] = $placeRgt;

    }else{
      if( !count($placeLft) && !count($placeRgt) ){
        unset($Filter['Filter']['Lft >=']);
        unset($Filter['Filter']['Rgt <=']);
      }
    }
    return place::Filter($Filter);
  }
  
  
  
  /*
    Получение страниц исходя из того скольок дочерних элементов есть у пейса
    @return Array
  */
  function GetPages(){
    return Utils::Paginator(
      $this->Children->Count,
      Site()->GetCountInPage()
    );
  }

  
  /*
    Получение всех дочерних элементов на все уровни вниз
    @return TListGroup
  */
  function GetPlaceChildrenAll(){
    $Filter = array( 
      'Filter' => array( 
        'Lft >' => $this->Lft,
        'Rgt <' => $this->Rgt,
      )
    );
    return place::Filter( $Filter );
  }



  static public function ByPlace( $Place, $c=-1 )
  {
    if( !isset( 
      place::$_R['ByPlace']
      ,place::$_R['ByPlace']['_'.$Place]

    )){
      
      $placeLft = array();
      $placeRgt = array();


      $Filter = array(
        'Filter' => array(
          'Place' => $this->Id,
          'Interprise' => Self()->Interprise->Id,
          'Lft >=' => $placeLft,
          'Rgt <=' => $placeRgt
        ),
        'Limit'=>$c
      );
      if( Self()->GetIsUser() ){
        $RulePlaceFilter = array(
          'Filter' => array(
            'User' => Self()->Id
          )
        );
        $RulePlaceResFilter = ruleplace::Filter( $RulePlaceFilter );
        if( $RulePlaceResFilter->Count ){
          foreach ($RulePlaceResFilter->Elements as $iRulePlace ) {
            $placeLft[] = $iRulePlace->Place->Lft;
            $placeRgt[] = $iRulePlace->Place->Rgt;
          }
        }

        $Filter['Filter']['Lft >='] = $placeLft;
        $Filter['Filter']['Rgt <='] = $placeRgt;

      }else{
        if( !count($placeLft) && !count($placeRgt) ){
          unset($Filter['Filter']['Lft >=']);
          unset($Filter['Filter']['Rgt <=']);
        }
      }

      place::$_R['ByPlace']['_'.$Place] = TObject2::Filter( new place(), $Filter);
    }
    return place::$_R['ByPlace']['_'.$Place];
  }

  
  
  
  
  
  /**
   * Получение дочерних элементов по страницам
   * 
   * @return \TListGroup
   */
  function GetChildrenByPage(){
    if( Site()->CountInPage < 0 ){
      return TListGroup::Init();
    }
    $Childs = $this->Children->GroupBy(Site()->CountInPage)->Elements;
    if( !count( $Childs ) ) return new TListGroup();
    if( isset( $Childs[ Site()->Page ] ) ){
      return $Childs[ Site()->Page ];
    }
    return $Childs[ count($Childs)-1 ];
  }
  
  
  
  
  
  
  /*
    Полученеи страниц для пагинации по дочерним плйсам
    @return Array
  */
  function GetPagesChildren( $is_all = false ){
    return Utils::Paginator(
      $this->Children->Count,
      Site()->GetCountInPage(),
      $is_all
    );
  }
  
  
  
  
  
  
  /*
    Активные параметры при показе данных
    @return TListGroup
  */
  function GetActiveParametersData(){
    
    /*
      Берем фильтр
      Если его нет то выводм все все есть
      Если есть то выводим данные по фильтру
    */
    $T = TListGroup::Init();

    if( $this->IsFilterParameterData ){
      $Parameters = array();

      $IssetsParametersByExtra = array();
      foreach( $this->AllParameters->Elements as $iParameter ){
        $IssetsParametersByExtra[ $iParameter->Id ] = true;
      }

      foreach( $this->FilterParameterData->Elements as $filterparamis ){
        if( !isset( $IssetsParametersByExtra[ $filterparamis->Parameter->Id ] ) ){
          continue;
        }
        $Parameters[ $filterparamis->Parameter->Id ] = $filterparamis->Parameter;
      }

      $T->SetFElements( array_values( $Parameters ) );
      return $T;
    }
    return $this->GetAllParameters();
  }
  
  
  
  
  /*
    Проверка есть фильтр или нет на приборе по данным
    @return Int
  */
  function GetIsFilterParameterData(){

    $SQL = Utils::Strip("
      select filterparamname.* 
      from filterparamname 
      left join place 
        on place.id = filterparamname.object_id 
      where place.lft <= {$this->Lft} 
        and place.rgt >= {$this->Rgt} 
        and filterparamname.interprise_id = " . Self()->Interprise->Id . "
        and place.interprise = " . Self()->Interprise->Id . "
    ");
    
    $t = filterparamname::Id(0);
    
    return TListGroup::Prepare( $SQL , $t )->Count;
  }
  
  
  
  
  
  
  
  
  /*
    Кеши фильтра параметров
  */
  private $CacheFilterParameterData = null;
  
    
  /*
    Получение фильтра параметрпов для построяния данных
    @return TListGroup
  */
  function GetFilterParameterData(){
    
    if( $this->CacheFilterParameterData ){
      return $this->CacheFilterParameterData->Parameters;
    }
    
    $this->CacheFilterParameterData = $this->GetFilterParameter('');
    return $this->CacheFilterParameterData->Parameters;
  }
  
  
  
  /*
    Построение массива из параметров для склеенного вывода
    @return Array
  */
  function GetHeadActiveParametersData( $after , $before ){
    $a = new HeadTableParameters( array_reverse($after) , array_reverse($before) , $this->GetActiveParametersData() , true );
    return $a->RowCol();
  }
  
  
  
  
  
  /**
   * Кеш TListGroup для параметров place 
   * @var type 
   */
  protected $CahceAllParametersTList = null;
  
  
  /*
    ВСЕ параметры у места
    @return TListGroup
  */
  function GetAllParameters(){
    
    if( $this->CahceAllParametersTList !== null ){
      return $this->CahceAllParametersTList;
    }
    
    if( !$this->IsTemp_place_mdh ){
      $this->Temp_place_mdh();
    }
    
    meter::Id(1)->Is;
    
    $AllParameters = array();
    
    $SQL = Utils::Strip("
      /* place.class.php */ select 
        parameter.*,
        if( meterparameter.id is null , metertypeparameter.is_use , meterparameter.isuse ) as is_use
      from place_mdh
      left join meter
        on meter.id = place_mdh.meter_id
      left outer join meterparameter
        on meterparameter.virtual_meter_id = meter.id
      left outer join metertypeparameter 
        on metertypeparameter.metertype_id = meter.type_id
      left join parameter
       on parameter.id = if( meterparameter.id is null , metertypeparameter.parameter_id , meterparameter.virtual_parameter_id )
      where parameter.id IS NOT NULL
      group by parameter.id
    ");
    
    
    $Res = DB::Select( $SQL );
    
    while( $Row = DB::Fetch( $Res ) ){
      $iParameter = parameter::SetFieldsFromObject( $Row );
      if( $iParameter->GetIsSys() && !$Row['is_use'] ){
        continue;
      }
      $AllParameters[ $Row['id'] ] = $iParameter;
    }
    
    $T = TListGroup::Init();
    $T->SetFElements( array_values( $AllParameters ) );
    
    $this->CahceAllParametersTList = $T;
    
    return $T;
  }
  
  
  /*
    Получение всех привязанных к этому месту приборов учета
    @return TListGroup
  */
  function GetMeters(){
    $SQL = Utils::Strip("
      SELECT meter.* FROM meterplace 
      LEFT JOIN meter 
        ON meter.id = meterplace.meter_id 
      where 
        meterplace.place_id = {$this->Id} and
        meter.converter_id > 0
      ORDER BY 
        meter.about,
        meter.num_serial
    ");
    
    $t = meter::Id(0);
    return TListGroup::Prepare( $SQL , $t );
  }
  
  
  
  
  
  
  
  
  
  
  /*
    Получение всех привязанных к этому месту приборов учета с ошибками
    @return TListGroup
  */
  function GetMetersErrorAll(){
    
    $meterplace_f = isset( meterplace::$FInit['InPath'] ) ? "meterplace.in_path = 0 AND" : "";
    
    $SQL = Utils::Strip("
        SELECT meter.* 
        FROM meterplace 
        LEFT JOIN place 
          on place.id = meterplace.place_id 
        left join meter 
          ON meter.id = meterplace.meter_id 
        where 
          place.lft >= {$this->Lft} and 
          place.rgt <= {$this->Rgt} and 
          {$meterplace_f} 
          place.interprise =".Self()->Interprise->Id." AND 
          meter.interprise = ".Self()->Interprise->Id." AND 
          meter.converter_id > 0 AND
          meter.id IS NOT NULL AND
          meter.is_ok = 0
        GROUP BY 
          place.id,meter.id 
        ORDER BY 
          place.lft,
          meter.about,
          meter.num_serial
      ");

    $t = meter::Id(0);
    return TListGroup::Prepare( $SQL , $t );
  }
  
  
  
  
  
  
  function GetMetersOkAll(){
    $meterplace_f = isset( meterplace::$FInit['InPath'] ) ? "meterplace.in_path = 0 AND" : "";
    
    $SQL = Utils::Strip("
        SELECT meter.* 
        FROM meterplace 
        LEFT JOIN place 
          on place.id = meterplace.place_id 
        left join meter 
          ON meter.id = meterplace.meter_id 
        where 
          place.lft >= {$this->Lft} and 
          place.rgt <= {$this->Rgt} and 
          {$meterplace_f} 
          place.interprise =".Self()->Interprise->Id." AND 
          meter.interprise = ".Self()->Interprise->Id." AND 
          meter.converter_id > 0 AND
          meter.id IS NOT NULL AND
          meter.is_ok = 1
        GROUP BY 
          place.id,meter.id 
        ORDER BY 
          place.lft,
          meter.about,
          meter.num_serial
      ");

    $t = meter::Id(0);
    return TListGroup::Prepare( $SQL , $t );
  }
  
  
  
  
  
  
  
  /*
    Получение всех привязанных к этому месту исключенных приборов учета
    @return TListGroup
  */
  function GetMetersExcludedAll(){
    $SQL = Utils::Strip("
      SELECT meter.* 
      FROM meterplace 
      LEFT JOIN place 
        on place.id = meterplace.place_id 
      left join meter 
        ON meter.id = meterplace.meter_id 
      where 
        place.lft >= {$this->Lft} and 
        place.rgt <= {$this->Rgt} and 
        place.interprise =".Self()->Interprise->Id." AND 
        meter.interprise = ".Self()->Interprise->Id." AND 
        meter.converter_id > 0 AND
        meter.is_excluded != 0 AND 
        meter.id IS NOT NULL 
      GROUP BY 
        place.id,meter.id 
    ");

    $t = meter::Id(0);
    return TListGroup::Prepare( $SQL , $t );
  }
  
  
  
  
  
  
  
  
  /*
    кеши для GetMetersAll
  */
  protected $MetersAllCache = null;
  
  
  /*
    Получение всех приборов с дочерних плейсов и свои
    @return TListGroup
  */
  function GetMetersAll(){
    if( $this->MetersAllCache === null ){
      
      
      $meterplace_f = isset( meterplace::$FInit['InPath'] ) ? "meterplace.in_path = 0 AND" : "";
      
      $SQL = Utils::Strip("
        SELECT meter.* 
        FROM meterplace 
        LEFT JOIN place 
          on place.id = meterplace.place_id 
        left join meter 
          ON meter.id = meterplace.meter_id 
        where 
          place.lft >= {$this->Lft} and 
          place.rgt <= {$this->Rgt} and 
          {$meterplace_f} 
          place.interprise =".Self()->Interprise->Id." AND 
          meter.interprise = ".Self()->Interprise->Id." AND 
          meter.converter_id > 0 AND
          meter.id IS NOT NULL 
        GROUP BY 
          place.id,meter.id 
        ORDER BY 
          place.lft,
          meter.about,
          meter.num_serial
      ");
      $t = new meter;
      $this->MetersAllCache = TListGroup::Prepare( $SQL, $t );
    }
    return $this->MetersAllCache;
  }
  
  
  
  
  
  
  
  
  function GetMetersAllByExtraType(){
      
      $meterplace_f = isset( meterplace::$FInit['InPath'] ) ? "meterplace.in_path = 0 AND" : "";
      
      
      // Если есть дополнительные типы то нужно сджойнить таблицы
      $Join_extra = "";
      $Where_extra = "";
      
      if( isset( $_GET['change_filter_ht'] ) ){
        
        // проверим валидность поля
        $Filter = array(
            'Filter' => array(
                'Ident' => $_GET['change_filter_ht']
            )
        );
        $ResExtra = meter_extra_type::Filter( $Filter );
        if( $ResExtra->Count ){
          $Join_extra = "left join meter_type ON meter.type_id = meter_type.id 
          left join meter_extra_type on meter_extra_type.id = meter.meter_extra_type_id
            or meter_type.meter_extra_type_id = meter_extra_type.id";
          $Where_extra = "meter_extra_type.ident = '{$ResExtra->First->Ident}' and ";
        }
      }
      
      $SQL = Utils::Strip("
        SELECT meter.* 
        FROM meterplace 
        LEFT JOIN place 
          on place.id = meterplace.place_id 
        left join meter 
          ON meter.id = meterplace.meter_id 
        {$Join_extra}
        where 
          place.lft >= {$this->Lft} and 
          place.rgt <= {$this->Rgt} and 
          {$meterplace_f} 
          {$Where_extra} 
          place.interprise =".Self()->Interprise->Id." AND 
          meter.interprise = ".Self()->Interprise->Id." AND 
          meter.converter_id > 0 AND
          meter.id IS NOT NULL 
        GROUP BY 
          place.id,meter.id 
        ORDER BY 
          place.lft,
          meter.about,
          meter.num_serial
      ");
      $t = new meter;
      $this->MetersAllCache = TListGroup::Prepare( $SQL, $t );
    return $this->MetersAllCache;
  }
  
  
  
  /*
    Возвращает все виды экспортов для плейсов
    @return TListGroup
  */
  function GetReports(){
    $Filter = array(
      'Filter' => array(
        'Objectrule' => $this->Objectrule->Id,
        'Interprise' => array( 0 , Self()->Interprise->Id )
      )
    );
    if( isset( $_GET['change_filter_ht'] ) ){
      $Filter["Filter"]["Namelink"] = array("xls","csv","html","xml",$_GET['change_filter_ht']); 
    }
    return reports::Filter( $Filter );
  }
  
  
  /*
    Получение пагинатора для дочерних элементов GetChildren
    return Array ( <num> => <int> , <name> => <string> , <is_active> => <bool> )
  */
  function GetPaginatorChildren(){
    return Utils::Paginator(
      $this->GetChildren()->Count ,
      Site()->CountInPage
    );
  }
  
  
  
  /*
    Получение Объекта ролей для места 
  */
  function GetObjectrule(){
    $Filter = array( 
      'Filter' => array( 
        'Ident' => 'Place'
      )
    );
    $ObjectruleFilter = objectrule::Filter( $Filter );
    if( $ObjectruleFilter->Count ){
      return $ObjectruleFilter->first;
    }
    return objectrule::Id(0);
  }
  
  
  
  
  /*
    Получение текущей даты на месте исходя из данных GET
    или времени сервера если данных нет
    @return DateTime
  */
  function GetDate(){
    $date = clone Site()->GetServerTime();
    if( $this->IsDateArchive ){
      $day    = isset( $_GET['day'] )   ? (int) $_GET['day']    : 0;
      $month  = isset( $_GET['month'] ) ? (int) $_GET['month']  : 0;
      $year   = isset( $_GET['year'] )  ? (int) $_GET['year']   : 0;
      $date->setDate(
        $year , $month , $day
      );
    }
    return $date;
  }
  
  
  
  
  
  
  /*
    Хранит в себе массив для быстрого перемещения и поиска
    по значениям
    array(
      meter_<ID>_parameter_<ID> => <num_row_reaource>
    )
  */
  protected $CacheArchive = null;
  
  
  
  
  /*
    Кеш на ресурс Для архива данных
  */
  protected $CacheArchiveResource = null;
  
  
  
  /*
    Кеш на ресурс Для последних данных
  */
  protected $CacheArchiveLastResource = null;
  
  
  
  /*
    Хранит в себе массив для быстрого перемещения и поиска
    по последним данным
    array(
      meter_<ID>_parameter_<ID> => <num_row_reaource>
    )
  */
  protected $CacheLastData = null;
  
  
  /*
    Кеш на ресурс Для последних данных
  */
  protected $CacheLastDataResource = null;
  
  
  
  
  /*
    Получение последних данных на приборах учета
    @return Bool
  */
  protected function SetCacheLastData(){
    
    /**
     * Если просмотр за архив то не показывать последние данные
     */
    if( !$this->IsToday() ){
      $this->CacheLastData = array();
      return true;
    }
    
    if( !$this->IsTemp_place_mdh ){
      $this->Temp_place_mdh();
    }
    
    $meterId = array();
    foreach( $this->MeterPlaceByPage->Elements as $iMeterPlace ){
      $meterId[ $iMeterPlace['Meter']->Id ] = $iMeterPlace['Meter']->Id; 
    }
    
    $parameterId = array();
    foreach( $this->ActiveParametersData->Elements as $iParameter ){
      $parameterId[ $iParameter->Id ] = $iParameter->Id;
    }
    
    meterlast::Id(1)->Is;
    
    $SQL = Utils::Strip("
      select 
        if( meterdatahour.id is null , meterlast.meterdatahour_id , meterdatahour.id ) as id,
        meterdatahour.year,
        meterdatahour.month,
        meterdatahour.day,
        meterdatahour.hour,
        if( meterparameter.meter_id is null , place_mdh.meter_id , meterparameter.virtual_meter_id ) as meter_id,
        if( meterparameter.parameter_id is null , meterdatahour.parameter_id , meterparameter.virtual_parameter_id ) as parameter_id,
        
        meterdatahour.tarif_id,
        meterdatahour.value,
        meterdatahour.loginfo_id,
        meterdatahour.add_time
      from place_mdh
      left outer join meterparameter
        on meterparameter.virtual_meter_id = place_mdh.meter_id
      left join meterlast
        on meterlast.meter_id = if( meterparameter.meter_id is null , place_mdh.meter_id , meterparameter.meter_id)
      left join meterdatahour
        on meterdatahour.id = meterlast.meterdatahour_id
      
      where meterdatahour.meter_id = meterlast.meter_id and meterdatahour.parameter_id = meterlast.parameter_id
    ");
    
    
    
    $Res = DB::Select( $SQL );
    
    $this->CacheArchiveLastResource = $Res;
    
    
    $i = 0;
    while( $Row = DB::Fetch( $this->CacheArchiveLastResource ) ){
      //~ var_dump($Row);
      if( 
        isset(  $this->CacheLastData[ $Row['parameter_id'] ] ) &&
        isset( $this->CacheLastData[ $Row['parameter_id'] ][ $Row['meter_id'] ] )
      ){
        continue;
      }
      
      $this->CacheLastData[ $Row['parameter_id'] ][ $Row['meter_id'] ] = $i;
      $i++;
      
      // $D = clone $this->Date;
      // $D->setDate( $Row['year'] , $Row['month'] , $Row['day'] );
      // //~ var_dump($D->getTimestamp() , $this->Date->getTimestamp());
      // if( $D->getTimestamp() >= $this->Date->getTimestamp() ){
      //   unset( $this->CacheLastData[ $Row['parameter_id'] ][ $Row['meter_id'] ] );
      // }
    }
    
    return true;
  }
  
  
  /*
    Наполнение кеша архива
      Format
        [ Parameter_Id ][ Meter_Id ] => Number_Row_In_Resource
    @return Bool
  */
  protected function SetCacheArchive(){
    
    
    //~ $Res = DB::MMQ( meterdatahour::Filter($Filter)->Sql );
    
    
    $Y = "place_mdh.year";
    $M = "place_mdh.month";
    $D = "place_mdh.day";
    
    if( !isset( $_GET['day'] ) && isset( $_GET['month'] ) ){
      $D = 0;
    }
    if( !isset( $_GET['month'] ) && isset( $_GET['year'] ) ){
      $D = 0;
      $M = 0;
    }
    
    
    $parameterId = array();
    foreach( $this->ActiveParametersData->Elements as $iParameter ){
      $parameterId[ $iParameter->Id ] = $iParameter->Id;
    }
    
    $Res = DB::Select(
      Utils::Strip("
        select 
           meterdatahour.id as id,
          meterdatahour.year,
          meterdatahour.month,
          meterdatahour.day,
          if( mp.meter_id is null , place_mdh.meter_id , mp.virtual_meter_id ) as meter_id,
          if( mp.parameter_id is null , metertypeparameter.parameter_id , mp.virtual_parameter_id ) as parameter_id,
    
          meterdatahour.tarif_id,
          meterdatahour.value,
          meterdatahour.loginfo_id,
          meterdatahour.add_time
          
        from place_mdh
    left join meter
          on meter.id = place_mdh.meter_id
        left outer join meterparameter as mp
          on mp.virtual_meter_id = meter.id
        left outer join metertypeparameter
          on metertypeparameter.metertype_id = meter.type_id
        left outer join meterdatahour
          on meterdatahour.meter_id = if( mp.meter_id is null , place_mdh.meter_id , mp.meter_id )
          and meterdatahour.parameter_id = if( mp.parameter_id is null , metertypeparameter.parameter_id , mp.parameter_id )
          and meterdatahour.year = {$Y}
      and meterdatahour.month = {$M}
      and meterdatahour.day = {$D}
      and meterdatahour.hour = 0
        where 
          if( mp.parameter_id is null , metertypeparameter.parameter_id , mp.parameter_id ) is not null
          and meterdatahour.id is not null 
      ")
    );
    
    
    $this->CacheArchiveResource = $Res;
    
    $this->CacheArchive = array();
    
    
    $i = 0;
    while( $Row = DB::Fetch( $this->CacheArchiveResource ) ){
      if( !isset( $this->CacheArchive[ $Row['parameter_id'] ] ) ){
        $this->CacheArchive[ $Row['parameter_id'] ] = array();
      }
      $this->CacheArchive[ $Row['parameter_id'] ][ $Row['meter_id'] ] = $i;
      $i++;
    }
    
    return true;
  }
  
  
  
  
  /*
    Наполняет свойство archiveMeterdatahourCache
    @return bool
  */
  function GetArchive( $iParameter , $iMeter ){
    
    //~ return meterdatahour::Id(0);
    
    if( !$this->IsTemp_place_mdh ){
      $this->Temp_place_mdh();
    }
    
    
    if( $this->CacheArchive === null ){
      $this->SetCacheArchive();
    }
    
    if( $this->CacheLastData === null ){
      $this->SetCacheLastData();
    }
    
    
    if( isset( $this->CacheArchive[ $iParameter->Id ][ $iMeter->Id ] ) ){
      DB::DataSeek( $this->CacheArchiveResource , $this->CacheArchive[ $iParameter->Id ][ $iMeter->Id ] );
      $data = DB::Fetch($this->CacheArchiveResource);
      $oMDH = meterdatahour::SetFieldsFromObject( $data );
      $oMDH->IS_LAST = false;
      return $oMDH;
    }elseif( isset( $this->CacheLastData[ $iParameter->Id ][ $iMeter->Id ] ) ){
      DB::DataSeek( $this->CacheArchiveLastResource , $this->CacheLastData[ $iParameter->Id ][ $iMeter->Id ] );
      $data = DB::Fetch($this->CacheArchiveLastResource);
      $oMDH = meterdatahour::SetFieldsFromObject( $data );
      $oMDH->IS_LAST = true;
      return $oMDH;
    }
    
    return meterdatahour::Id(0);
  }
  
  
  
  
  
  
  /*
    Постраничная навигация для отчета
    @return Array
  */
  function GetPaginatorArchive(){
    return Utils::Paginator(
      $this->MetersAllByExtraType->Count,
      Site()->CountInPage
    );
  }
  
  
  
  
  
  
  
  
  
  /*
    Получение meterdatahour из ресурса
    @return meterdatahour
  */
  function GetMeterdatahourByMeterParameter( $Row ){
    
    $oMeter = $Row['meter'];
    $oParameter = $Row['parameter'];
    
    if( $this->archiveMeterdatahourCache === null ){
      $this->GetArchive();
    }
    
    $ukey = "meter_" . $oMeter->Id . "_parameter_" . $oParameter->Id;
    
    if( isset( $this->archiveMeterdatahourCache[ $ukey ] ) ){
      DB::DataSeek( $this->ResourceArchive , $this->archiveMeterdatahourCache[ $ukey ] );
      
      unset( $this->archiveMeterdatahourCache[ $ukey ] );
      
      $row = DB::Fetch($this->ResourceArchive);
      
      return meterdatahour::SetFieldsFromObject( $row );
      
    }
    
    return meterdatahour::Id(0);
  }
  
  
  
  /*
    Проверяет является ли запрашиваемая дата через GET архивной
    @return Boolean
  */
  function GetIsDateArchive(){
    return $this->StepArchive !== false;
  }
  
  
  /*
    Возвращает понимаение какой шаг используется в архиве
    @return String month day hour // какой из них равен 0
    Если значения не за какой то архив то вернет FALSE
  */
  function GetStepArchive(){
    if( isset( $_GET['year'] ) ){
      if( isset( $_GET['month'] ) ){
        if( isset( $_GET['day'] ) ){
          return "hour";
        }
        return "day";
      }
      return "month";
    }
    return false;
  }
  
  
  /*
    Формирование ссылки назад во времени
    @return String
  */
  function GetPrevDateLink(){
    return $this->GetLinkArchive().$this->PrevDate->format( $this->DateFormatLink );
  }
  
  /*
    Получение предыдущей даты
    @return DateTime
  */
  function GetPrevDate(){
    $date = clone $this->Date;
    $date->modify('-1 day');
    return $date;
  }
  
  
  /*
    Получение удобно читаемой даты для пользователя
    @return String
  */
  function GetPrevDateTitle(){
    return $this->PrevDate->format( $this->DateFormat );
  }
  
  
  /*
    показ текущей даты на месте
    @return String
  */
  function GetDateLink(){
    return $this->Date->format( $this->DateFormat );
  }
  
  
  /*
    Формирование ссылки вперед во времени
    @return String
  */
  function GetNextDateLink(){
    return $this->LinkArchive.$this->NextDate->format( $this->DateFormatLink );
  }
  
  
  
  
  
  
  
  
  
  
  /*
    Хранит в себе массив из кол-ва дочерних элементов (мест на этом месте)
  */
  public $CacheCountChildrenPlace = null; 
  
  /*
    Получение кол-ва дочерних мест на текущем месте
    ВНИМАНИЕ
    Чтобы получить кол-во надо обращатся к методу через родителя места на которое
    необходимо узнать
  
    @param place
    @return Int
  */
  function PlacesCount( $oPlace ){
    if( $this->CacheCountChildrenPlace === null ){
      $SQL = Utils::Strip("
        select 
          place.id,
          count(place_child.id ) as c
        from place
        left join place as place_child 
          on place.id = place_child.place_id
        where place.place_id = {$this->Id}
        and place.interprise = ".Self()->Interprise->Id."
        and place_child.id IS NOT NULL
        group by place.id
      ");
      $res = DB::Select($SQL);
      while( $row = DB::Fetch($res)){
        $this->CacheCountChildrenPlace[ $row['id'] ] = $row['c'];
      }
    }
    return isset($this->CacheCountChildrenPlace[ $oPlace->Id ])
          ? $this->CacheCountChildrenPlace[ $oPlace->Id ]
          : 0;
  }
  
  
  
  /*
    Проверка что можно удалить место или нет
  */
  function GetIsDelete(){
    return !$this->Place->PlacesCount($this) &&
          !$this->Place->ChildrenReadMetersByStatuses($this)['all'];
  }
  
  
  
  /*
    Получение следующей даты
    @return DateTime
  */
  function GetNextDate(){
    $date = clone $this->Date;
    $date->modify('+1 day');
    return $date;
  }
  
  
  
  /**
   * Если просмотр идет за текущее число
   * @return boolean
   */
  function IsToday(){
    return Site()->GetDateServer()->format('Ymd') == $this->GetDate()->format('Ymd');
  }
  
  
  
  
  /*
    Получение удобно читаемой даты для пользователя
    @return String
  */
  function GetNextDateTitle(){
    return $this->NextDate->format( $this->DateFormat );
  }
  
  
  
  /*
    Общий шаблон ссылки
    @return String
  */
  static function MainLink(){
    return "/place/";
  }
  
  
  /*
    Ссылка на место
    @return Int
  */
  function GetLink(){
    $extra_type = "";
    if( isset( $_GET['change_filter_ht'] ) ){
      switch( $_GET['change_filter_ht'] ){
        case "AquaCounter" :
          $extra_type = "water/";
          break;
        case "HeatCounter" :
          $extra_type = "heat/";
          break;
      }
    }
    return $this->GetBaseLink() . $extra_type;
  }
  
  
  /** 
   * Базовая ссылка на место
   * 
   * @return string
   */
  function GetBaseLink(){
    return self::MainLink() . "{$this->Id}/";
  }
  
  
  /*
    Ссылка на удаление места
    @return String
  */
  function GetLinkDel(){
    return $this->Link . "delete/";
  }
  
  
  /*
    Ссылка на удаление места
    @return String
  */
  function GetLinkDelete(){
    return $this->LinkDel;
  }
  
  
  
   
  
  
  
  /*
    Ссылка на просмотр данных
    @return String
  */
  function GetLinkData(){
    return $this->Link;
  }
  
   
  
  
  
  /*
    Ссылка на просмотр файлов
    @return String
  */
  function GetLinkFiles(){
    return $this->Link."files/";
  }
  
   
  

  /**
   * Ссылка на просмотр архивов
   * 
   * Если на текущее число то ссылка будет вида /place/<:id>/report/<:id_report>/
   * Если за архивное, то 
   * /place/<:id>/archive/<:year>/<:month>/<:day>/report/<:id_report>/
   * /place/<:id>/archive/<:year>/<:month>/report/<:id_report>/
   * /place/<:id>/archive/<:year>/report/<:id_report>/
   * 
   * см. /place/.htaccess
   * 
   * @return string
   */
  function GetLinkReports(){
    
    $sufix = "";
    
    $append_sufix = array();
    
    foreach ( array( 'year' , 'month' , 'day' ) as $interval ){
      if( isset( $_GET[ $interval ] ) ){
        $append_sufix[] = $_GET[ $interval ];
      }
    }
    
    $sufix .= implode( '/' , $append_sufix );
    
    if( empty( $append_sufix ) ){
      return $this->GetLink()."reports/";
    }
    return $this->GetLinkArchive()."{$sufix}/reports/";
  }
  
  
  
  
  /*
    Ссылка на редактирование места
    @return String
  */
  function GetLinkEdit(){
    return $this->Link . "edit/";
  }
  
  
  
  
  /*
    Ссылка на редактирование места
    @return String
  */
  function GetLinkDelFile(){
    return $this->Link . "delete_file/";
  }
  
  
  
  /*
    Ссылка на соглашение удалить файл
    @return String
  */
  function GetLinkFileApply(){
    return $this->Link . "delete_file_apply/";
  }
  
  
  
  /*
    Ссылка на добавление дочернего места
    @return String
  */
  function GetLinkAdd(){
    return $this->Link . 'add/';
  }

  
  /*
    Ссылка на просмотр архива
    @return String
  */
  function GetLinkArchive(){
    return $this->GetLink() . "archive/";
  }

  
  /*
    Ссылка на просмотр прав
    @return String
  */
  function GetLinkRule(){
    return $this->Link . "rule/";
  }
  
  
  
  
  /*
    Ссылка на просмотр архива
    @return String
  */
  function GetLinkTree(){
    return $this->Link . "tree/";
  }
  
  
  
  
  /*
    Получение связи прибор - плейс для указанной страницы
    UPD; У прибора учета появится новое свойство Place.
    
    @return TListGroup
  */
  function GetMeterPlaceByPage(){
    
    
    // Формируем поля с префиксом для приборов учета
    
    $f = array();
    
    
    foreach( explode(',',meter::$SqlFields) as $field ){
      preg_match( '/^(.*)\`([a-z|_]*)\`$/',$field, $m);
      if( isset( $m[2] ) ) $f[] = $field . " as meter_" . $m[2];
    }
    
    foreach( explode(',',place::$SqlFields) as $field ){
      preg_match( '/^(.*)\`([a-z|_]*)\`$/',$field, $m);
      if( isset( $m[2] ) ) $f[] = $field . " as place_" . $m[2];
    }
    
    
    // Формируем поля с префиксом для плейсов
    
    $Limit = ( Site()->CountInPage * Site()->Page ) . ",".Site()->CountInPage;
    
    
    $meterplace_f = isset( meterplace::$FInit['InPath'] ) ? 'meterplace.in_path = 0 AND' : '' ;
    
    
    // Если есть дополнительные типы то нужно сджойнить таблицы
    $Join_extra = "";
    $Where_extra = "";

    if( isset( $_GET['change_filter_ht'] ) ){

      // проверим валидность поля
      $Filter = array(
          'Filter' => array(
              'Ident' => $_GET['change_filter_ht']
          )
      );
      $ResExtra = meter_extra_type::Filter( $Filter );
      if( $ResExtra->Count ){
        $Join_extra = "left join meter_type ON meter.type_id = meter_type.id 
        left join meter_extra_type on meter_extra_type.id = meter.meter_extra_type_id
          or meter_type.meter_extra_type_id = meter_extra_type.id";
        $Where_extra = "meter_extra_type.ident = '{$ResExtra->First->Ident}' and ";
      }
    }
    
    
    $SQL = Utils::Strip("
      SELECT 
        " . implode( ',' , $f ) . " 
        FROM meterplace 
        LEFT JOIN place 
          on place.id = meterplace.place_id 
        left join meter 
          ON meter.id = meterplace.meter_id 
        {$Join_extra}
        where 
          place.lft >= {$this->Lft} and 
          place.rgt <= {$this->Rgt} and 
          {$meterplace_f}
          {$Where_extra}
          place.interprise =".Self()->Interprise->Id." AND 
          meter.interprise = ".Self()->Interprise->Id." AND 
          meter.converter_id > 0 AND 
          meter.id IS NOT NULL 
        GROUP BY 
          place.id,meter.id 
        ORDER BY 
          place.place_id,
          place.lft,
          meter.about,
          meter.num_serial 
        LIMIT {$Limit}
    ");
        
        
    $Output = array();
    
    $Res = DB::Select( $SQL );
    while( $Row = DB::Fetch( $Res ) ){
      
      $m_f = array();
      $p_f = array();
      
      foreach( $Row as $name_field => $value_field ){
        if( preg_match( '/^meter_(.*)$/' , $name_field , $m ) ){
          $m_f[ $m[1] ] = $value_field;
        }elseif( preg_match( '/^place_(.*)$/' , $name_field , $m ) ){
          $p_f[ $m[1] ] = $value_field;
        }
      }
      
      
      $Metertype = metertype::Id($m_f['type_id']);
      $class = "meter";
      if( $Metertype->IsVirtual ){
        $class = "vmeter";
      }elseif( $Metertype->IsCalc ){
        $class = "cmeter";
      }
      
      $Output[] = array(
        'Meter' => $class::SetFieldsFromObject( $m_f ),
        'Place' => place::SetFieldsFromObject( $p_f ),
      );
    }
    
    $R = TListGroup::Init();
    $R->SetFElements( $Output );
    
    return $R;
  }
  
  
  
    /*
    Создание объекта из массива данных
    @return meterdatahour
  */
  static function SetFieldsFromObject( $aValues = array() ){
    $t = new place();
    $t = $t->Init($t,$aValues);
    return $t;
  }

  
  
  
  
  
  
  
  
  /*
    Вывод в виде строки пути до этого места
      ВКЛЮЧАЯ ЭТО МЕСТО!
    @param String разделитель
    @return String
  */
  function PathToUpString( $separator = " / " ){
    $paths = array();
    foreach( $this->ParentAll->Elements as $iPlace ){
      $paths[] = $iPlace->Title;
    }
    $paths[] = $this->Title;

    return implode( $separator , $paths ) . $separator;
  }
  
  
  
  
  
  
  
  
  
  
  /*
    Все активные параметры на плейсе
    @return TListGroup
  */
  function GetActiveParameters(){
    return $this->AllParameters;
  }
  
  
  
  
  
  /*
    Получение всех родителей
    @return TListGroup
  */
  function GetParentAll(){
    $Filter = array(
      'Filter' => array(
        'Lft <=' => $this->Lft,
        'Rgt >=' => $this->Rgt,
        'Interprise' => $this->Interprise->Id
      ),
      'AddWhere' => ' AND lft != 0 AND rgt != 0',
      'Order' => array(
        'Lft' => 'ASC'
      )
    );
    return place::Filter( $Filter );
  }
  
  
  
  
  /*
    Не рпазобранный стек переменных из старой версии
  */
  static $LastInterprisePoint = 0;
  static $LastLvlPoint = 0;
  static $PointerConsole = true;

  /**
   * Запись LFT RGT началась?
   *
   * @var boolean 
   */
  static $replaceToNull = false;

  /**
   * Глобальное переназначение всем плейсам Lft Rgt
   * 
   * @param int $interprise
   * @param int $place
   * @param int $pointer
   * @return boolean
   */
  static function ReplaceLftPgt( $interprise = 0 , $place = 0 , &$pointer = 0 ){

    if( !place::$replaceToNull ) {

      DB::Log("globalReplacePointer");

      DB::Select( 'START TRANSACTION' );
      foreach(place::All()->GroupBy(100)->Elements as $iPlaceGroup){
        foreach($iPlaceGroup->Elements as $iPlace){
          $iPlace->Lft = 0;
          $iPlace->Rgt = $iPlace->Id;
        }
      }
      place::$replaceToNull = true;
    }
    
    
    $Filter = array(
      'Filter' => array(
        'Place' => $place,
        'Interprise' => $interprise
      ),
      'Order' => array(
        'Title' => 'ASC'
      )
    );
    
    if( self::$LastInterprisePoint != $interprise ){
      self::$LastInterprisePoint = $interprise;
    }
    
    self::$LastLvlPoint++;
    
    foreach( place::Filter( $Filter )->GroupBy(100)->Elements as $iPlaceGroup ){
      foreach($iPlaceGroup->Elements as $iPlace){
        
        $pointer++;
        
        $iPlace->Lft = $pointer;
        
        $pointer = self::ReplaceLftPgt( $interprise , $iPlace->Id , $pointer );
        
        $pointer++;
        
        self::$LastLvlPoint--;
        
        $iPlace->Rgt = $pointer;

      }
      $iPlaceGroup->ElementsFreeMem;
    }
    
    
    if( $place == 0 ) {
      foreach(place::All()->GroupBy(100)->Elements as $iPlaceGroup){
        foreach($iPlaceGroup->Elements as $iPlace){
          $iPlace->Save();
        }
      }
      DB::Select( "COMMIT" );
    }
    
    return $pointer++;
  }
  
  
  static $PointerSeparator='';
  
  
  function GetLft(){
    if( $this->_F['Lft']['ready'] && !$this->_F['Lft']['value'] ){
      self::globalReplacePointer(Self()->Interprise->Id);
    }
    if( !$this->_F['Lft']['ready'] ){
      TObject2::_DataLoadSql($this);
    }
    return $this->_F['Lft']['value'];
  }

  
  static function ReplacePointer( $added_place ){

    // Rgt родителя станет нашим lft

    $Lft = $added_place->Place->Rgt;
    $Rgt = $Lft +1;

    // echo "Update place lft = lft+2,rgt=rgt+2 where lft >= {$Lft}";

    DB::Query("Update place set lft = lft+2,rgt=rgt+2 where lft >= {$Lft}");
    DB::Query("Update place set rgt=rgt+2 where rgt >={$Lft} and lft < {$Lft}");
    DB::Query("Update place set lft={$Lft},rgt={$Rgt} where id ={$added_place->Id}");

  }


  
  /**
   * Перезапись указателей мест Lft Rgt
   * 
   * @param interprise $interprise
   * @param place $place
   * @param integer $pointer
   * @return boolean
   */
  static function globalReplacePointer( $interprise = 0 , $place = 0 , & $pointer = 0 ){
    
    if( count( func_get_args() ) == 1 ){
      TObject2::CleanAllObjectsByClass('place');
      self::$LastInterprisePoint = 0;
      self::$LastLvlPoint = 0;
      self::$PointerConsole = true;
      self::$replaceToNull = false;
    }

    if( !place::$replaceToNull ) {

      DB::Log("globalReplacePointer");

      DB::Select( 'START TRANSACTION' );
      foreach(place::All()->GroupBy(100)->Elements as $iPlaceGroup){
        foreach($iPlaceGroup->Elements as $iPlace){
          $iPlace->Lft = 0;
          $iPlace->Rgt = $iPlace->Id;
        }
      }
      place::$replaceToNull = true;
    }
    
    $str = self::$PointerSeparator;
    
    $Filter = array(
      'Filter' => array(
        'Place' => $place,
        'Interprise' => $interprise
      ),
      'Order' => array(
        'Title' => 'ASC'
      )
    );
    
    $debug = isset($_GET['debug']);
    
    if( self::$LastInterprisePoint != $interprise && $debug ){
      if(!self::$PointerConsole) echo "<br><br><br>Interprise : {$interprise}\n";
      else "\n\n\nInterprise : {$interprise}\n";
      self::$LastInterprisePoint = $interprise;
      flush();
    }
    
    if( $debug ){
      self::$LastLvlPoint++;
      if(!self::$PointerConsole) {
        echo str_repeat( $str , self::$LastLvlPoint * 2 ) . " <b><sup style=\"font-size:10px;color:green;\">$pointer</sup></b>&nbsp;{$place}&nbsp;--> <br>\n";
      }else{
        echo str_repeat( $str , self::$LastLvlPoint * 2 ) . " $pointer [ {$place} ] --> \n";
      }
    }
    
    if( !place::Filter( $Filter )->Count && $place == 0 && $debug ){
      echo "<b>Нет мест</b><br>\n";
    }
    
    //~ var_dump(place::Filter( $Filter )->Sql);
    //~ echo "\n\n";
    
    foreach( place::Filter( $Filter )->GroupBy(100)->Elements as $iPlaceGroup ){
      
      foreach($iPlaceGroup->Elements as $iPlace){
        
        $pointer++;
        
        $iPlace->Lft = $pointer;
        
        $pointer = self::globalReplacePointer( $interprise , $iPlace->Id , $pointer );
        
        $pointer++;
        
        if( $debug ){
          self::$LastLvlPoint--;
          if(!self::$PointerConsole) {
            echo str_repeat( $str , self::$LastLvlPoint * 2 ) . " <--&nbsp;{$place}&nbsp;<b><sup style=\"font-size:10px;color:red;\">$pointer</sup></b> <br>\n";
          }else{
            echo str_repeat( $str , self::$LastLvlPoint * 2 ) . " <--[ {$place} ] $pointer\n";
          }
        }
        
        $iPlace->Rgt = $pointer;

      }
      $iPlaceGroup->ElementsFreeMem;
    }
    
    
    if( $place == 0 ) {
      foreach(place::All()->GroupBy(100)->Elements as $iPlaceGroup){
        foreach($iPlaceGroup->Elements as $iPlace){
          $iPlace->Save();
        }
      }
      DB::Select( "COMMIT" );
      TObject2::CleanAllObjectsByClass('place');
    }
    
    return $pointer++;
  }
  
  
  
  
  
  /*
    Чтение фалов закрепелнных за местами
    @return Array
  */
  function GetReadDirFiles(){
    $dir = Site()->GetPathPlaceFiles( $this );
    $F = array();
    if ($handle = @opendir($dir)) {
        while (false !== ($entry = readdir($handle))) {
          if( $entry == "." || $entry == ".." ) continue;
          $F[] = $entry;
        }
        closedir($handle);
    }
    return $F;
  }
  
  
  /*
    Получение публичного пути до папаки с файлами мест
    @return String
  */
  function GetDirFilesPub(){
    return str_replace(
      $_SERVER['DOCUMENT_ROOT'] 
      , '' ,
      Site()->GetPathPlaceFiles( $this )
    );
  }
  
  
  
  
  
  
  
  
  
  
  
  
  public function UpdateByRequest(){
    if( isset( $_REQUEST['PlaceId'] ) && $_REQUEST['PlaceId'] == $this->Id ){
      unset($_REQUEST['PlaceId']);
    }
    TObject2::StaticUpdateByRequest($this);
  }
  
  
  
  
  public function UpdateByArray($A){
    if( isset( $A['PlaceId'] ) && $A['PlaceId'] == $this->Id ){
      unset($A['PlaceId']);
    }
    TObject2::StaticUpdateByArray($this,$A);
  }
  
  
  
  
  function Delete() {
    if( $this->Interprise->Id != Self()->Interprise->Id ){
      return false;
    }
    if( !$this->GetIsDelete() ){
      return false;
    }
    $this->RelateMeters->DeleteAll();
    TObject2::StaticDelete($this);
  }
  
  
  
  
  /**
   * Вставка без перезаписи указателей?
   *
   * @var boolean 
   */
  public static $AddByNotReplaceLftRgt = false;
  
  
  
  /*
    Добавление нового плейса
    @return place
  */
  static public function AddByArray($A){
    $t = new place();
    if( !isset( $A['InterpriseId'] ) ){
      $A['InterpriseId'] = Self()->Interprise->Id;
    }
    TObject2::StaticAddByArray($t,$A);
    
    if( !self::$AddByNotReplaceLftRgt ){
      // self::globalReplacePointer( $t->Interprise->Id );
      self::ReplacePointer( $t );
    }
    
    return $t;
  }
  
  
  /*
    Добавление нового плейса
    @return place
  */
  static public function AddByRequest(){
    $t = new place();
    if( !isset( $_REQUEST['InterpriseId'] ) ){
      $_REQUEST['InterpriseId'] = Self()->Interprise->Id;
    }
    TObject2::StaticAddByArray($t,$_REQUEST);
    
    if( !self::$AddByNotReplaceLftRgt ){
      // self::globalReplacePointer( $t->Interprise->Id );
      self::ReplacePointer( $t );
    }

    return $t;
  }
  
  
  /*
    Получение всех мест
    @return TListGroup
  */
  static public function All( $c=-1 )
  {

    $placeLft = array();
    $placeRgt = array();

    if( !isset( 
      place::$_R['All']
    )){
      $Filter = array( 
        'Filter' => array( 
          'Interprise' => Self()->Interprise->Id,
          'Lft >=' => $placeLft,
          'Rgt <=' => $placeRgt
        )
      );


      if( Self()->GetIsUser() ){
        $RulePlaceFilter = array(
          'Filter' => array(
            'User' => Self()->Id
          )
        );
        $RulePlaceResFilter = ruleplace::Filter( $RulePlaceFilter );
        if( $RulePlaceResFilter->Count ){
          foreach ($RulePlaceResFilter->Elements as $iRulePlace ) {
            $placeLft[] = $iRulePlace->Place->Lft;
            $placeRgt[] = $iRulePlace->Place->Rgt;
          }
        }

        $Filter['Filter']['Lft >='] = $placeLft;
        $Filter['Filter']['Rgt <='] = $placeRgt;

      }else{
        if( !count($placeLft) && !count($placeRgt) ){
          unset($Filter['Filter']['Lft >=']);
          unset($Filter['Filter']['Rgt <=']);
        }
      }

      place::$_R['All'] 
      = TObject2::Filter( new place() , $Filter);
    }
    return place::$_R['All'];
  }
  
  
  
  
  static public function Filter($aParams = null , $etc = array()){
    $t=new place();
    if( !isset( $aParams['Filter']['Interprise'] ) ){
      $aParams['Filter']['Interprise'] = Self()->Interprise->Id;
    }
    return TObject2::Filter($t,$aParams);
  }
  
  
  
  
  
  
    
  
  
  /**
   * Получение всех приборов исходя из их дополнительного типа
   * 
   * @param string $type
   * @return TListGroup
   */
  protected function GetMetersByExtraType( $type ){
    $Interprise = Self()->Interprise->Id;
    $SQL = Utils::Strip("
  select
    `meter`.*
  from meterplace
  left join place 
    on place.id = meterplace.place_id
  left join meter 
    on meter.id = meterplace.meter_id
  left join meter_type
    on meter.type_id = meter_type.id
  left join meter_extra_type 
    on meter.meter_extra_type_id = meter_extra_type.id
    or meter_type.meter_extra_type_id = meter_extra_type.id
  where meter_extra_type.ident = '{$type}'
  and meter.interprise = {$Interprise}
  and meter.converter_id > 0
  and place.lft >= {$this->Lft}
  and place.rgt <= {$this->Rgt}
");
  
    $t = meter::Id(0);
    return TListGroup::Prepare($SQL , $t);
  }
  
  
  
  
  
  /**
   * Получение кол-ва электросчетчиков на месте
   * 
   * @return TListGroup
   */
  function GetElectricityMeters(){
    return $this->GetMetersByExtraType( "ElectricityCounter" );
  }
  
  
  
  
  /**
   * Получение водосчетчиков на месте
   * 
   * @return TListGroup
   */
  function GetWaterMeters(){
    return $this->GetMetersByExtraType( "AquaCounter" );
  }
  
  
  
  
  /**
   * Получение теплосчетчиков на месте
   * 
   * @return TListGroup
   */
  function GetHeatMeters(){
    return $this->GetMetersByExtraType( "HeatCounter" );
  }
  
  

}
