<?php
/**
 * Smarty plugin
 *
 * @package    Smarty
 * @subpackage PluginsModifier
 */

/**
 * Smarty date_format modifier plugin
 * Type:     modifier<br>
 * Name:     date_format<br>
 * Purpose:  format datestamps via strftime<br>
 * Input:<br>
 *          - string: input date string
 *          - format: strftime format for output
 *          - default_date: default date if $string is empty
 *
 * @link   http://www.smarty.net/manual/en/language.modifier.date.format.php date_format (Smarty online manual)
 * @author Monte Ohrt <monte at ohrt dot com>
 *
 * @param string $string       input date string
 * @param string $format       strftime format for output
 * @param string $default_date default date if $string is empty
 * @param string $formatter    either 'strftime' or 'auto'
 *
 * @return string |void
 * @uses   smarty_make_timestamp()
 */
function smarty_modifier_date_format($string, $format = null, $default_date = '', $formatter = 'auto')
{
    if ($format === null) {
        $format = Smarty::$_DATE_FORMAT;
    }
    /**
     * require_once the {@link shared.make_timestamp.php} plugin
     */
    require_once(SMARTY_PLUGINS_DIR . 'shared.make_timestamp.php');
    if ($string != '' && $string != '0000-00-00' && $string != '0000-00-00 00:00:00') {
        $timestamp = smarty_make_timestamp($string);
    } elseif ($default_date != '') {
        $timestamp = smarty_make_timestamp($default_date);
    } else {
        return;
    }
    if ($formatter == 'strftime' || ($formatter == 'auto' && strpos($format, '%') !== false)) {
        if (DS == '\\') {
            $_win_from = array('%D', '%h', '%n', '%r', '%R', '%t', '%T');
            $_win_to = array('%m/%d/%y', '%b', "\n", '%I:%M:%S %p', '%H:%M', "\t", '%H:%M:%S');
            if (strpos($format, '%e') !== false) {
                $_win_from[] = '%e';
                $_win_to[] = sprintf('%\' 2d', date('j', $timestamp));
            }
            if (strpos($format, '%l') !== false) {
                $_win_from[] = '%l';
                $_win_to[] = sprintf('%\' 2d', date('h', $timestamp));
            }

            $format = str_replace($_win_from, $_win_to, $format);
        }
        
        /*
          Сделано специально для вывода почасовых данных на приборе учета
        */
        
        if( $format == "%custom_hour" ){
          $_D = new DateTime();
          $_D->setTimestamp($timestamp);
          $_D->modify("+1 hour");
          return strftime("%H:00", $timestamp) . "-" . strftime("%H:00", $_D->getTimestamp());
          
          
        /*
          Сделано для вывода месячных данных
        */
        }elseif( $format == "%custom_month" ){
          switch( ceil( strftime("%m", $timestamp) ) ){
            case 1    : return strftime("%Y январь"   , $timestamp);
            case 2    : return strftime("%Y февраль"  , $timestamp);
            case 3    : return strftime("%Y март"     , $timestamp);
            case 4    : return strftime("%Y апрель"   , $timestamp);
            case 5    : return strftime("%Y май"      , $timestamp);
            case 6    : return strftime("%Y июнь"     , $timestamp);
            case 7    : return strftime("%Y июль"     , $timestamp);
            case 8    : return strftime("%Y август"   , $timestamp);
            case 9    : return strftime("%Y сентябрь" , $timestamp);
            case 10   : return strftime("%Y октябрь"  , $timestamp);
            case 11   : return strftime("%Y ноябрь"   , $timestamp);
            case 12   : return strftime("%Y декабрь"  , $timestamp);
          }
        }elseif( $format == "%custom_day" ){
          $d = strftime("%d", $timestamp);
          switch( ceil( strftime("%m", $timestamp) ) ){
            case 1    : $m = 'январь';    break;
            case 2    : $m = 'февраль';   break;
            case 3    : $m = 'март';      break;
            case 4    : $m = 'апрель';    break;
            case 5    : $m = 'май';       break;
            case 6    : $m = 'июнь';      break;
            case 7    : $m = 'июль';      break;
            case 8    : $m = 'август';    break;
            case 9    : $m = 'сентябрь';  break;
            case 10   : $m = 'октябрь';   break;
            case 11   : $m = 'ноябрь';    break;
            case 12   : $m = 'декабрь';   break;
          }
          
          return $d . " " . $m;
        }
        return strftime($format, $timestamp);
    } else {
        return date($format, $timestamp);
    }
}
