if(typeof graph_data == 'undefined')
  var graph_data = "График потребления энергии за ";

if(typeof no_graph_data == 'undefined')
  var no_graph_data = "Нет данных для построения графика";

if(typeof graph_in == 'undefined')
  var graph_in = "За";

if(typeof graph_hour == 'undefined')
  var graph_hour = "час";

/**
 * Формирование заголовка для графика на местах
 * 
 * @returns {String}
 */
function chart_title(){
  let a = new Date(); 
  
  let r1 = new RegExp(/^(.*)\/archive\/([0-9]*)\/([0-9]*)\/([0-9]*)\/$/);
  let r2 = new RegExp(/^(.*)\/archive\/([0-9]*)\/([0-9]*)\/$/);
  let r3 = new RegExp(/^(.*)\/archive\/([0-9]*)\/$/);
  
  let prefix = graph_data;
  
  let w = window.location.pathname;
  
  if( r1.test( w ) ){
    let r = r1.exec( w );
    return prefix + r[2] + "-" + r[3] + "-" + r[4]; 
  }else if( r2.test( w ) ){
    let r = r2.exec( w );
    return prefix + r[2] + "-" + r[3]; 
  }else if( r3.test( w ) ){
    let r = r3.exec( w );
    return prefix + r[2]; 
  }
  return ""; 
}



Array.prototype.min = function(){
    return Math.min.apply( null, this.reduce(function(res,item){
      if( item ){
        res.push(item);
      }
      return res;
    },[]) );
};


/**
 * Рисует график для мест
 * 
 * @param {Object} series_1_2
 * @param {Object} series_1
 * @param {Object} series_2
 * @param {String} id_container
 * @returns {Object}
 */

var Chart = null

function place_chart( series_1_2 , series_1 , series_2 , id_container ){
  var series = []

  if( series_1_2 && series_1_2.data ){
    series.push( series_1_2 )
  }
  if( series_1 && series_1.data.length ){
    series.push( series_1 )
  }
  if( series_2 && series_2.data.length  ){
    series.push( series_2 )
  }

  Chart = Highcharts.chart( id_container ,  {
    colors: ['#7cb57c','#a8b4bd','#2677b5'],
    chart: {
        renderTo: '#container_highcharts',
        type: 'areaspline'
    },
    title: {
        text: ''
    },
    subtitle: {
        text: ''
    },
    xAxis: {
        allowDecimals: false,
        labels: {
            formatter: function () {
                return this.value; // clean, unformatted number for year
            }
        },
        tickInterval : 4
    },
    yAxis: {
        title: {
            text: typeof YAXIS_PLACE_PARAMETERS == 'undefined' ? "" : YAXIS_PLACE_PARAMETERS
        },
        // type: 'logarithmic',
        labels: {
            formatter: function () {
                return this.value.toFixed(0) ;
            },
        },
        min : [series_1_2.data.min(),series_1.data.min(),series_2.data.min()].min()
    },
    tooltip: {
        pointFormat: '{series.name} <br><b>{point.y}</b><br/>' + graph_in + ' {point.x} ' + graph_hour
    },
    title: {
        text : ( !series_1_2.length && !series_1.data.length && !series_2.data.length ) 
                ? no_graph_data
                : chart_title()
    },
    plotOptions: {

        series : {
            events : {
              show : function(e){
                rerender_minYaxis()
              },
              hide : function(e){
                rerender_minYaxis()
              },
            }
        },

        areaspline: {
          //~ stacking: 'normal',
          lineColor: '#666666',
          lineWidth: 1,
          pointStart: 1,
          pointInterval: 1, // hourly data
          marker: {
            enabled: true,
            symbol: 'circle',
            radius: 3,
            states: {
              hover: {
                enabled: true
              }
            }
          }
        }
    },

     series: series

  });
  return Chart;
}

function rerender_minYaxis(){
  Highcharts.each(Highcharts.charts, function(p, i) {
                 var  min = null
                  for (var i = 0; i < p.series.length ; i++) {
                    if( p.series[i].visible ){
                      if( min == null || min > p.series[i].dataMin ) {
                        min = p.series[i].dataMin
                      }
                    }
                  }
                  p.yAxis[0].update({
                    min:  min
                  })
                  p.render()
                })
}