define([
  "backbone",
  "places",
  "place_modal_add",
  "place_modal_edit",
  "meters",
  "meter_places"
],function(
  Backbone,
  places,
  place_modal_add,
  place_modal_edit,
  meters,
  meter_places
){
  
  return Backbone.View.extend({
    el          : '#place_children_table',
    collection  : places,
    template    : '#children_item_tpl',
    
    
    
    /*
      @param int Идентификатор плейса из URL
    */
    initialize : function( id ){
      
      this.get_id = id
      
      var self = this
      
      // при добавлении нового дочернего элемента
      this.listenTo( this.collection , 'add_children' , function(change_model){
        
        var self = this

        if( change_model.get('id') == self.get_id ) return false
        
        this.blocked_el()
        
        
        $.get( '/tpl/?tpl=place_children_item' , 
                { data : [change_model.get('id')] } , 
                function(data){
                  
                  let isset_tr = $(self.$el).find( '#'+ $(data).attr('id') )
                  
                  if( !isset_tr.length ){
                    $(self.$el).find('[data-droped]')
                              .last()
                              .after( $(data) )
                  }else{
                    $( isset_tr ).replaceWith( $(data) )
                  }
                  
                  self.unblocked_el()
                  
                }
              )
      })
      
      
      this.listenTo( places , 'change_model' , function( change_model ){
        
        let data = { data : [change_model.get('id')] }
        
        $.get( '/tpl/?tpl=place_children_item' , 
          data , 
          function(data){
            
            let isset_tr = $(self.$el).find( '#'+ $(data).attr('id') )
            
            $( isset_tr ).replaceWith( $(data) )
            
          }
        )
        
        
        $.get( '/tpl/?tpl=place_one_info' , 
          data , 
          function(data){
            
            $('#place_item_info').find('tbody').replaceWith(
              $(data).find('tbody')
            )
            
          }
        )
        
      })
      
      
    },
    
    
    blocked_el : function(){
      $(this.$el).addClass('disable_div_mini')
    },
    unblocked_el : function(){
      $(this.$el).removeClass('disable_div_mini')
    },
    
    
    events : {
      
      /*
        Добавление дочернего элемента в плейс
      */
      'click .short_add_place' : function( t ){
        
        this.blocked_el()
        
        let id = $(t.target).closest('tr')
                            .attr('id')
                            .replace(/^([a-z|\_]*)([0-9]*)$/,function(_,name,id){
                              return id
                            })
        var self = this
        
        
        /*
          Получаем информацию о плейсе на который хотим добавлять
        */
        places.fetch({
          data : {
            filter : "id:" + id
          },
          
          // отдаем модально окно
          success : function(a){
            new place_modal_add( Number( self.get_id ) || 0 , Number( id ) || 0 , a.last() )
            self.unblocked_el();
          },
          
          // отдаем сообщение о ошибке
          error : error_handler
        })
        return false; // запрещаем клик
        
      },
      
      
      
      
      /*
        Добавление в текущий элемент
      */
      'click #add_place_modal' : function(t){
        
        this.blocked_el()
        
        var id = Number( this.get_id ) || 0
        
        var self = this;
        
        /*
          Так как скорее всего этого плейса нет то его надо загрузить
        */
        
        let place_clone = places.clone().reset()
        
        place_clone.fetch({
          data : {
            filter : "id:" + id
          },
          success : function(a){
            let model = !id ? new a.model : a.last()
            new place_modal_add( id , id , model )
            self.unblocked_el();
          },
          error : error_handler
        })
        
        return false;
        
      },
      
      
      
      
      /*
        Редактирование плейса
      */
      'click .edit_children_place' : function( t ){
        
        this.blocked_el()
        
        let id = $(t.target).closest('tr')
                            .attr('id')
                            .replace(/^([a-z|\_]*)([0-9]*)$/,function(_,name,id){
                              return id
                            })
        var self = this
        
        /*
          Получаем информацию о плейсе на который хотим добавлять
        */
        places.fetch({
          data : {
            filter : "id:" + id
          },
          
          // отдаем модально окно
          success : function(a){
            new place_modal_edit( a.last() )
            self.unblocked_el();
          },
          
          // отдаем сообщение о ошибке
          error : error_handler
        })
        
        return false;
      },
      
      
      
      
      
      
      
      /*
        Редактирование приборов учета в плейсе
      */
      'click .short_add_meter_to_place' : function( t ){
        
        this.blocked_el()
        
        var self = this
        
        let id = $(t.target).closest('tr')
                            .attr('id')
                            .replace(/^([a-z|\_]*)([0-9]*)$/,function(_,name,id){
                              return id
                            })
        
        places.fetch({
          data : {
            filter : "id:"+id
          },
          success : function(pl){
            modal( pl.last() )
          }
        })
        
        
        
        /*
          Модальное окно
        */
        var modal = function( model ){
          
          var div = $('<div/>')
                              
          $(div).jstree({
            core : {
              data : {
                url : function(node){
                  if( node.id == '#' ){
                    return "/api/?object=converter&tree"
                  }else{
                    return "/api/?object=meter&tree&filter=converter:" + node.id.replace("converter_tree_",'');
                  }
                },
              },
              'themes': {
                            'name': 'proton',
                            'responsive': true,
              },
              'check_callback' : function(operation, node, node_parent, node_position, more){
                  return true;
              },
            },
            "checkbox": {
                "three_state": false
            },
            plugins : ["ui","checkbox"]
          }).bind('loaded.jstree',function(e,data){
            
            self.unblocked_el();
            
            var pos_data = {
              my: "top", at: "top", of: window, collision: "fit"
            }
            
            $(window).resize(function(){
              $(div).dialog("widget").position( pos_data )
            });
            $(div).dialog("widget").position( pos_data )
            
            meters.fetch({
              data : {
                filter : "place:" + id
              },
              success  :function( a ){
                a.toArray().map(function( item ){
                  data.instance.open_node( "converter_tree_" + item.get('converter') ,function(){
                    data.instance.select_node( "meter_tree_" + item.get('id') )
                  })
                })
              },
          
              // отдаем сообщение о ошибке
              error : error_handler
            })
            
          }).bind('changed.jstree',function(e,data){
            
            
            if(data.event){
              switch( data.action ){
                case 'select_node' :
                  let m = meter_places.model
                  var y = new m({
                    meter : data.node.id.replace('meter_tree_',''),
                    place : id
                  })
                  
                  y.save({}).then(function(a){
                    var m = places.model
                    places.trigger('add_children' , new m({id : a.place}) )
                  })
                break;
                case 'deselect_node' :
                  meter_places.fetch({
                    data : {
                      filter : "meter:" + data.node.id.replace('meter_tree_','') 
                      + ';place:'+id
                    },
                    success : function(d){
                      var _a = d
                      if( d.last().get('id') ){
                        d.last().destroy({dataType: "text",success : function(){
                          var m = places.model
                          places.trigger('add_children' , new m({id : id}) )
                        }})
                      }
                    },
          
                    // отдаем сообщение о ошибке
                    error : error_handler
                  })
                break;
              }
            }
          }).bind('load_node.jstree',function(e,data){
            data.node.children.map(function(id){
              if( !id.match(/^meter/) ){
                $('#' + id).find('.jstree-checkbox').remove()
              }
            },data)
          }).bind('open_node.jstree',function(e,data){
            if( !data.node.id.match(/^meter/) ){
              $('#' + data.node.id + ' > a .jstree-checkbox').remove()
            }
            data.node.children.map(function(id){
              if( !id.match(/^meter/) ){
                $('#' + id).find('.jstree-checkbox').remove()
              }
            },data)
          }).dialogYesNo({
            title : model.get('title'),
            isOk : function(t){
              $(t).dialog('destroy')
            }
          })
        }
        
        
        
        
        return false;
      },
      
      
      
      
      
      
      /*
        Удаление плейсов
      */
      'click .delete_children_place' : function( t ){
        
        
        
        var self = this;
        
        if( $(t.target).closest('td').find('a').hasClass('text-muted') ){
          $('<div/>').append(
            'Нельзя удалить. Есть вложенные элементы. Сначала удалите их.'
          ).dialogWarning({
            title : 'Ошибка'
          })
        }else{
          
          self.blocked_el()
          
          let id = $(t.target).closest('tr')
                              .attr('id')
                              .replace('children_place_','')
          
          places.fetch({
            data : {
              filter : "id:"+id
            },
            success : function(pl){
              var model = pl.last()
              
              $('<div/>').append(
                "Вы уверены что хотите удалить?"
              ).dialogYesNo({
                title : model.get('title'),
                isOk : function(t){
                  model.destroy({dataType: "text",success : function(a,b){
                    if( b == 'droped' ){
                      $('#children_place_' + model.get('id')).remove()
                    }
                  }})
                  $(t).dialog('destroy')
                }
              })
              
              self.unblocked_el()
              
            },
          
            error : error_handler
          })
          
        }
        
        return false;
        
        
        
        
      }
      
    }
  })
    
})