define([
  "backbone",
  "archive_add_modal",
  "archives",
  "archive_add_meter_modal",
  "archive_delete_modal",
  "upload",
  "iframe"
],function(
  Backbone,
  archive_add_modal,
  archives,
  archive_add_meter_modal,
  archive_delete_modal
){
  
  
  const PREFIX = "panel_archive_"
  
  
  
  
  
  function load_files( view ){
    var url = window.location.pathname;
    var res = $('#fileupload').fileupload({
        url: url,
        dataType: 'json',
        done: function (e, data) {
            $('#progress').addClass('hide')
        },
        progressall: function (e, data) {
          $('#progress').removeClass('hide')
            var progress = parseInt(data.loaded / data.total * 100, 10);
            $('#progress .progress-bar').css(
                'width',
                progress + '%'
            );
            
        },
        error : function (jqXHR, textStatus, errorThrown) {
          
          $('#progress').addClass('hide')
          
          
          switch( jqXHR.responseText ){
            case 'invalid_format' :
              var tmpl = _.template($('#item_danger_files').html())
              
              $('#results').append(
                tmpl({
                  why : "Не верный формат файла"
                })
              )
              
              $('.progress-bar').attr('class','progress-bar progress-bar-danger')
              
            break;
            case 'not_load' :
              var tmpl = _.template($('#item_danger_files').html())
              
              $('#results').append(
                tmpl({
                  why : "Не получилось загрузить"
                })
              )
              
              $('.progress-bar').attr('class','progress-bar progress-bar-danger')
              
            break;
            default:
              
              $('.reporting_alert').remove();
              $('#page-wrapper').prepend(
                $('<div/>').addClass('alert alert-success reporting_alert').text('Архив успешно загружен.')
              )
              
              var data = { data : [ jqXHR.responseText ] }
              $.get(
                '/tpl/?tpl=archive_custom_list_item',
                data,
                function( tpl ){
                  $( '#accordion' ).prepend( $( tpl ) )
                  $('#accordion').find('.panel').first().effect('bounce')
                }
              )

              
            break;
          }
        }
    })
  }
  
  
  

  return Backbone.View.extend({
  
  
    initialize : function(){
      
      
      load_files( this );
      
      var self = this
      
      
      /*
        Редактирование модули
      */
      this.listenTo( archives , 'change_model' , function( model ){
        
        var data = { data : [ model.get('id') ] }
        
        $.get(
          '/tpl/?tpl=archive_item',
          data,
          function( tpl ){
            
            var tpl = $(tpl)
            
            if( model.get('error') ){
              $( tpl ).addClass('panel-danger')
            }
            $( '#' + $(tpl).attr('id') ).replaceWith( $( tpl ) )
          }
        )
        
      })
      
      
      
      /*
        Добавление модули
      */
      this.listenTo( archives , 'add_model' , function( model ){
        
        var data = { data : [ model.get('id') ] }
        
        $.get(
          '/tpl/?tpl=archive_item',
          data,
          function( tpl ){
            
            
            var tpl = $(tpl)
            
            $( '#accordion' ).append( $( tpl ) )
            $( tpl ).find('.meter_edit').trigger('click')
          }
        )
        
      })
      
      
      
      
      
      /*
        Удаление модули
      */
      this.listenTo( archives , 'drop_model' , function( model ){
        
        $.when( $('#' + PREFIX + model.get('id')).slideDown(300) ).then(function( el ){
          $(el).remove()
        })
        
      })
      
      
      $( this.$el ).find('.progress').each(function( i , pr ){
        console.log(pr)
        if( !$(pr).closest('.panel').find('.file_name').length ){
          self.hatiko( $(pr).closest('.panel').attr('id').replace( PREFIX , '' ) )
        }
      })
      
      
    },
    
    
    /*
      Таймеры постоянно отсылающие запросы на сервер
      см. hatiko
    */
    intervalAr : {},
    
    
    /*
      модели за которыми будем наблюдать
      см. hatiko
    */
    modelAr : {},
    
    
    /*
      Постоянно спрашивает сервер о состоянии модели.
      Как только модель меняется то показываем это пользователю.
    */
    hatiko : function( id ){
      
      var prefix_hatiko = "l_"
      
      var self = this
      
      var id = id || 0
      
      if( typeof this.intervalAr[ prefix_hatiko + id ] != "undefined" ){
        return false;
      }
      if( typeof this.modelAr[ prefix_hatiko + id ] != "undefined" ){
        return false;
      }
      
      
      this.modelAr[ prefix_hatiko + id ] = new archives.model({ id : id })
      
      this.modelAr[ prefix_hatiko + id ].on('change' , function(){
        if( Number(this.get('procent')) == 0 && Number(this.get('status')) == 0 ){
          clearInterval( self.intervalAr[ prefix_hatiko + id ] )
        }
        archives.trigger('change_model' , this )
      })
      
      this.intervalAr[ prefix_hatiko + id ] = setInterval(function(){
        archives.fetch({
          data : {
            filter : "id:" + id
          },
          success : function( data ){
            self.modelAr[ prefix_hatiko + id ].set( data.first().toJSON() )
          }
        })
      } , 3000)
      
      
    },
    
    
  
  
    events : {
      'click #add_new_archive' : function(){
        new archive_add_modal();
        return false;
      },
      
      'click .edit_archive' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        new archive_add_modal( id );
        return false;
      },
      
      'click .delete_archive' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        new archive_delete_modal( id );
        return false;
      },
      
      'click .meter_edit' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        new archive_add_meter_modal( id );
        return false;
      },
      
      
      'click .debug_report' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        $.get('/archive/' + id + '/report/',function( res ){
          $('.reporting_alert').remove();
          if( res == "file_not_exists" ){
            $('#page-wrapper').prepend(
              $('<div/>').addClass('alert alert-danger reporting_alert').text('Отчет не был отправлен.')
            )
          }else{
            $('#page-wrapper').prepend(
              $('<div/>').addClass('alert alert-success reporting_alert').text('Отчет отправлен.')
            )
          }
        })
        return false;
      },
      
      
      'click .archive_stoped' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        var self = this
        $.ajax({
          url: '/archive/' + id + '/stop/',
          beforeSend: function(){
            self.hatiko( id )
          },
          complete : function(a,b,c){
            if( a.responseText != "ok" ){
              self.modelAr[ "l_" + id ].set({
                procent : 0,
                status : 0,
                error : false
              })
            }
          }
        })
        return false;
      },
      
      
      'click .run_dump' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        
        var self = this
        
        var e = e
        
        if( !isNaN(id) && id != 0 ){
          
          $.ajax({
            url: '/archive/run/',
            data : {
              id : id
            },
            beforeSend: function(){
              self.hatiko( id )
            },
            complete : function(a,b,c){
              //~ if( a.responseText != "ok" ){
                //~ self.modelAr[ "l_" + id ].set({
                  //~ procent : 0,
                  //~ status : 0,
                  //~ error : true
                //~ })
              //~ }
            }
          })
        }else{
          
          $.ajax({
            url: '/archive/run/',
            data : {
              file : $( e.target ).closest('.panel').find('.file_name').text()
            },
            //~ beforeSend: function(){
              //~ self.hatiko( id )
            //~ },
            complete : function(a,b,c){
              var data = { data : [ a.responseText ] }
              
              $.get(
                '/tpl/?tpl=archive_item',
                data,
                function( tpl ){
                  
                  var tpl = $(tpl)
                  
                  $( e.target ).closest('.panel').replaceWith( $( tpl ) )
                }
              )
              
            }
          })
        }
        return false;
      },
      
      
      
      'click .droped_files' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        
        $.when( $( e.target ).closest('.panel').slideUp(300) ).then(function( r ){
          $.get(
            '/archive/' + id + '/delete/',
            {
              name : $( e.target ).closest('.panel').find('.file_name').text()
            },
            function( res ){
              if( res == 'droped' ){
                $( e.target ).closest('.panel').remove()
              }else{
                $( e.target ).closest('.panel').slideDown(300)
              }
            }
          )
        })
        
      },
      
      
      
      'click .run_archive' : function( e ){
        var id = $( e.target ).closest('.panel').attr('id').replace( PREFIX , '' )
        var self = this
        $( e.target ).closest('.btn-group').dropdown('toggle')

        $.ajax({
          url: '/archive/' + id + '/start/',
          beforeSend: function(){
            self.hatiko( id )
          },
          complete : function(a,b,c){
            
            $( '#page_error').empty().append(function(){
              switch( a.responseText ){
                case 'okyear_invalid' :
                  return $('<div/>').addClass('alert alert-danger').text('Не верный год')
                break;
              }
            })
            
            self.modelAr[ "l_" + id ].set({
              procent : 0,
              status : 0,
              error : a.responseText == 'ok'
            })
          }
        })
        return false;
      }
      
    },
  
    el : '#container_archive_list',
  
  
  
  });

  
})