define([
  "backbone",
  "mails",
  "places",
  "validator",
  "clockpicker",
  "datepicker",
  "jstree"
],function(
  Backbone,
  mails,
  places
){

  
  $.fn.datepicker.dates['en'] = {
      days: ["Воскресенье", "Понедельник", "Вторник", "Среда", "Четверг", "Пятница", "Суббота"],
      daysShort: ["Вос", "Пон", "Вто", "Wed", "Thu", "Fri", "Sat"],
      daysMin: ["Вс", "Пн", "Вт", "Ср", "Чт", "Пт", "Сб"],
      months: ["Январь", "Февраль", "Март", "Апрель", "Май", "Июнь", "Июль",
               "Август", "Сентябрь", "Октябрь", "Ноябрь", "Девабрь"],
      monthsShort: ["Янв", "Фев", "Мар", "Апр", "Май", "Июн", "Июл", "Авг", "Сен", "Окт", "Ноя", "Дек"],
      today: "Сегодня",
      clear: "Очистить",
      format: "mm/dd/yyyy",
      titleFormat: "MM yyyy",
      weekStart: 1
  };

  
  const TEST = true
  
  
  /**
   * Неактивная плашка
   * @type {String}
   */
  const DISABLE_I_CLASS     = "milestone-default";

  /**
   * Активная плашка
   * @type {String}
   */
  const ACTIVE_I_CLASS      = "milestone-primary";

  /**
   * Пройденая плашка
   * @type {String}
   */
  const NOTACTIVE_I_CLASS   = "milestone-info";
  
  
  /**
   * Кол-во шагов для создания уведомления
   * @type {Number}
   */
  const COUNT_STEP = 5;
  
  
 
  /**
   * Проверка на валидность адреса почты
   * @param  {String} email Почта для проверки
   * @return {Bool}
   */
  function validateEmail(email) {
      var re = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,15})+$/;
      return re.test(email);
  }
  
  
  
  

  /**
   * Открытие модального окна
   * @param  {Int} id   идентификатор почтового уведомления
   * @param  {Object} tpl объект модального окна
   * @return {Bool}
   */
  function show_modal( id , tpl ){
   
    var id  = id  || 0,
        tpl = tpl || $('<div/>')
    
    mails.fetch({
      data : {
        filter : "id:" + id
      },
      success : function( data ){
        
        if( data.first() ){
          String( data.first().get('crontab') ).replace(/^([0-9|\*]*) ([0-9|\*]*) (.*)$/ , function( _ , minutes , hours ){
            $('.custom_time_res_hour').text( hours )
            $('.custom_time_res_min').text( minutes )
          })
          
          String( data.first().get('crontab') ).replace(/^(.*) ([0-9|,]*)$/ , function( _ , any , d_weeks ){
            
            if(d_weeks){
              $('#custom_day_week_res').val(d_weeks)
              $('.custom_week_day').addClass('active')
            }
          })
          
          String( data.first().get('crontab') ).replace(/^(.*) (.*) (.*) (.*) (.*)$/ , function( _ , m, h, d, m, w ){
            
            if( w == "*" ){
              if(d != "*"){
                $('#my_hidden_input').val(d)
                $('.custom_day').addClass('active')
              }else{
                $('.every_day').addClass('active')
              }
            }
          })
          
          
          $('#Name').val( data.first().get('name') )
          
          data.first().get('listmail').split("\n").map(function( email ){
            
            if( !email ) return false;
            
            $.when( $('#append_email').before(
              $('<a/>').addClass('list-group-item').append(
                $('<span/>').text( email).addClass('valid_email')
              ).append(
                $('<button/>').addClass('close drop_email').attr({
                  type : "button"
                }).text('×')
              ).append(
                $('<button/>').addClass('close edit_email').attr({
                  type : "button"
                }).append(
                  $('<i/>').addClass('fa fa-pencil').css({
                    'font-size' : '14px',
                    padding : '4px 11px'
                  })
                )
              )
            ) ).then(function( a ){
              $(a).prev().slideDown(300)
            })
          })
        }
        
        
        $('#listing_emails').on('keyup','input',function(e){
          switch( e.originalEvent.keyCode ){
            case 13 :
              $( e.target ).closest('a').find('.saved_email').trigger('click')
              if( validateEmail( $( e.target ).val() ) ){
                $('#append_email').click()
              }
            break;
          }
        })
        
        
        $(tpl).trigger('callback_step_1')
        $(tpl).trigger('callback_step_2',[true])
        $(tpl).trigger('callback_step_3')
        $(tpl).trigger('callback_step_4')
        $(tpl).trigger('callback_step_5')
        $(tpl).trigger('callback_step_5')
      }
    })
    return false;
  }
  

  
  
  
  
  
  /*
    Событие-проверка того что список email не пуст
  */
  function is_empty_listmail(){
    
    var count_mail = $('#main_mail').find('.valid_email,a input.form-control').length
    
    if( !$('#empty_email').length && !count_mail ){
      $('#append_email').before(
        $('<a/>').addClass(
          'list-group-item disabled'
        ).attr({
          id : "empty_email"
        }).text('Нет ни одного Email')
      )
    }else{
      if( count_mail ){
        $('#empty_email').remove()
      }
    }
  }
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  /*
    Включение режима на каждый час
    @returns {Bool} false
  */
  function every_time( tpl ){
    $( '.every_time' ).closest('.toggle_body').find('a').each(function( i , a ){
      if( !$(a).hasClass('every_time') ){
        $(a).removeClass('active')
      }
    })
    $( '.every_time' ).toggleClass('active')
    $(tpl).trigger('callback_step_2')
    return false;
  }
  
  
  
  
  
  
  
  
  
  
  /*
    Выбор собственного времени
    
    @param Event
    @param {DOM}
    
    @return false
  */
  function custom_time( e , tpl  ){
    
    var tpl = tpl || null
    
    // Если это клик по инпуту который должен 
    if( $( e.target )[0].tagName == $('<input/>')[0].tagName ) {
      
      $(e.target).on('change' , function( e ){
        if( !$(this).val().concat("").match(/([0-9]*):([0-9]*)/) ){
          $('.custom_time_res').addClass('hide')
        }
        $(this).val().concat("").replace(/([0-9]*):([0-9]*)/ , function( _, h, m){
          $('.custom_time_res_hour').text( h )
          $('.custom_time_res_min').text( m )
        })
        $(tpl).trigger('callback_step_2')
      })
      
      // Если родитель активный то показываем часы
      // Если нет то скрываем
      
      if( $( e.target ).closest('a').hasClass('active') ){
        $('.clockpicker').clockpicker('show')
      }else{
        $('.clockpicker').clockpicker('destroy')
      }
      $(tpl).trigger('callback_step_2')
      return false;
    }
    
    $( '.custom_time' ).closest('.toggle_body').find('a').each(function( i , a ){
      if( !$(a).hasClass('custom_time') ){
        $(a).removeClass('active')
      }
    })
    
    $( '.custom_time' ).addClass('active')
    
    
    if( $( '.custom_time' ).hasClass('active') ){
      
      $.when( $('.clockpicker').clockpicker({
        beforeDone: function( e ) {
          $(tpl).trigger('callback_step_2')
        },
      }) ).then(function(){
        $('.clockpicker').find('input').trigger('click',[ tpl ])
      })
    }else{
      $('.clockpicker').find('input').trigger('click',[ tpl ])
    }
    
    return false;
  }
  
  
  
  
  
  /*
    Добавление нового поля для написания email
    @return false
  */
  function append_mail(){
    $.when( $(this).before(
      $('<a/>').addClass('list-group-item').append(
        $('<div/>').addClass('input-group').append(
          $('<input/>').addClass('form-control')
        ).append(
          $('<span/>').addClass('input-group-addon alert-success saved_email').append(
            $('<i/>').addClass('fa fa-save')
          ).css({
            cursor : "pointer"
          })
        ).append(
          $('<span/>').addClass('input-group-addon alert-danger drop_email').append(
            $('<i/>').addClass('fa fa-times')
          ).css({
            cursor : "pointer"
          })
        )
      ).hide()
    ) ).then(function( a ){
      $(a).prev().slideDown(300).find('input').focus().on('focusout' , function(){
        if( $(this).val().trim() == "" ){
          $(this).closest('a').find('.drop_email').trigger('click')
        }
      })
      $('.modal').trigger('callback_step_1')
    })
    
    return false;
  }
  
  
  
  
  
  
  
  
  /*
    Сохранение email из поля в строку
    @param DOM tpl
    @return false
  */
  function saved_email( tpl ){
    let email = $(this).closest('.list-group-item').find('.form-control').val()
            
    if( !validateEmail( email ) || !email.trim() ){
      $(this).closest('.list-group-item').addClass('has-error')
      return false;
    }
    
    $(this).closest('.list-group-item').removeClass('has-error').empty().append(
      $('<span/>').text( email).addClass('valid_email')
    ).append(
      $('<button/>').addClass('close drop_email').attr({
        type : "button"
      }).text('×')
    ).append(
      $('<button/>').addClass('close edit_email').attr({
        type : "button"
      }).append(
        $('<i/>').addClass('fa fa-pencil').css({
          'font-size' : '14px',
          padding : '4px 11px'
        })
      )
    )
    
    $(tpl).trigger('callback_step_1')
    
    return false;
  }
  
  
  
  
  
  
  /*
    Удаление Email
    @param Event
    @param tpl
    @return false
  */
  function drop_email( e , tpl ){
    
    var tpl = tpl || $('<div/>')
    
    $( e.target ).closest('.list-group-item').slideUp(300,function(){
      $(e.target).closest('a').remove()
      $(tpl).trigger('callback_step_1')
    })
    return false;
  }
  
  
  
  
  
  
  /*
    Редактивароение email
    @param DOM 
    @returns Bool false
  */
  function edit_email( tpl ){
    
    var tpl = tpl
    
    let email = $(this).closest('.list-group-item').find('span').text()
    $(this).closest('.list-group-item').empty().append(
      $('<div/>').addClass('input-group').append(
        $('<input/>').addClass('form-control').val( email )
      ).append(
        $('<span/>').addClass('input-group-addon alert-success saved_email').append(
          $('<i/>').addClass('fa fa-save')
        ).css({
          cursor : "pointer"
        })
      ).append(
        $('<span/>').addClass('input-group-addon alert-danger drop_email').append(
          $('<i/>').addClass('fa fa-times')
        ).css({
          cursor : "pointer"
        })
      )
    )
    
    $(tpl).trigger('callback_step_1')
    
    return false;
  }
  
  
  
  
  
  
  /*
    Проверка валидности 1 шага
    @return Bool
  */
  function is_valid_step_1(){
    return $('#listing_emails').find('.list-group-item').find('.valid_email').length &&
           $('#Name').val().trim() != ""
  }
  
  
  
  
  
  /*
    Действия при валидности\не валидности первого шага
    return false
  */
  function callback_step_1(){
    let valid = is_valid_step_1()
                
    let f_1 = valid ? 'removeClass' : 'addClass'
    let f_2 = valid ? 'addClass' : 'removeClass'
    
    var steps = valid ? [ 2 ] : [2,3,4]
    
    steps.map(function(i){
      $('#step_' + i).find('.circle')[f_1]( DISABLE_I_CLASS )[f_2]( NOTACTIVE_I_CLASS )
      $('#step_' + i).find('.header')[
        valid ? 'removeClass' : 'addClass'
      ]( 'text-muted' );
      if( valid ){
        $('#step_' + i).find('.form-group')[
          valid ? 'removeClass' : 'addClass'
        ]('hide')
      }
    })
    
    
    $('.modal').trigger('is_empty_listmail');
  }
  
  
  
  
  
  
  /*
    Проверка валидности второго окна
    @return Bool
  */
  function is_valid_step_2( is_first ){
    /*
      Если это не первая проверка и нет ни одного активного пункта. Не валидно!
    */
    if( !is_first && !$('#step_2').find('a.active').length ){
      // Сюда надо написать скрытие всех шагов
      $('#step_3').find('.form-group').addClass('hide')
      $('#container_tree').addClass('hide')
      $('#step_5').find('select').addClass('hide')
      return false;
    }
    
    if( $('.custom_time').hasClass('active') ){
      let custom_time = $('.custom_time_res_hour').text() != "" && $('.custom_time_res_min').text() != ""
      let null_date = $('.custom_time_res_hour').text() == "-" && $('.custom_time_res_min').text() == "-"
      return custom_time && !null_date;
    }else{
      // 1. хз почему не через триггер
      // 2. хз зачем это вообще тут
      //$('.every_time').addClass('active')
    }
    return $('#step_2').find('a.every_time').hasClass('active');
  }
  
  
  
  /*
    Коллбек второго окна
    @return false
  */
  function callback_step_2( e , is_first ){
    
    var is_first = is_first || null
    
    /*
      Если есть выставленное время то значит выставляли
    */
    if( is_first ){
    
      let custom_time = $('.custom_time_res_hour').text() != "" && $('.custom_time_res_min').text() != ""
      
      let null_date = $('.custom_time_res_hour').text() == "-" && $('.custom_time_res_min').text() == "-"
      
      $('.custom_time_res')[
        custom_time && !null_date ? 'removeClass'
                    : 'addClass'
      ]('hide')
      
      $('.custom_time')[
        custom_time && !null_date ? 'addClass'
                    : 'removeClass'
      ]('active')
      
      $('.every_time')[
        !custom_time && null_date ? 'addClass'
                    : 'removeClass'
      ]('active')
      
      $('.clockpicker').find('input').val(
        custom_time && !null_date 
        ? $('.custom_time_res_hour').text().concat(":").concat( $('.custom_time_res_min').text() )
        : ""
      )
    }
    
    let valid = is_valid_step_2( is_first ) && is_valid_step_1()
    
    let f_1 = valid ? 'removeClass' : 'addClass'
    let f_2 = valid ? 'addClass' : 'removeClass'
                
    var steps = valid ? [ 3 ] : [3,4,5]
    
    steps.map(function(i){
      $('#step_' + i).find('.circle')[f_1]( DISABLE_I_CLASS )[f_2]( NOTACTIVE_I_CLASS )
      $('#step_' + i).find('.header')[
        valid ? 'removeClass' : 'addClass'
      ]( 'text-muted' )
      if( valid ){
        $('#step_' + i).find('.form-group')[
          valid ? 'removeClass' : 'addClass'
        ]('hide')
      }
    })
  }
  
  
  
  
  
  
  
  
  /*
    Проверка валидности ввода дней для отчетов
    @returns {Bool}
  */
  function is_valid_step_3(){
    if( $('.every_day').hasClass('active') ){
      return true;
    }else if( $('.custom_week_day').hasClass('active') ){
      return $( '#custom_day_week_res' ).val().trim() != "";
    }else if( $('.custom_day').hasClass('active') ){
      return $('#my_hidden_input').val()
    }
    
    return false;
  }
  
  
  
  
  
  
  /*
    Коллбек третьего шага
    @returns {Bool} false
  */
  function callback_step_3(){
    
    let valid = is_valid_step_3() &&
                is_valid_step_2() &&
                is_valid_step_1()
    
    let i = 4;
    
    let f_1 = valid ? 'removeClass' : 'addClass'
    let f_2 = valid ? 'addClass' : 'removeClass'
    
    var steps = valid ? [ 4 ] : [4,5]
    
    steps.map(function(i){
      $('#step_' + i).find('.circle')[f_1]( DISABLE_I_CLASS )[f_2]( NOTACTIVE_I_CLASS )
      $('#step_' + i).find('.header')[
        valid ? 'removeClass' : 'addClass'
      ]( 'text-muted' )
      if( valid ){
        $('#step_' + i).find('#container_tree')[
          valid ? 'removeClass' : 'addClass'
        ]('hide')
      }
    })

    
    if( valid &&  typeof $('#container_tree').data('jstree') == 'undefined' ){
      
      
      var opened = [];
      
      // Открывает дерево мест
      
      $('#container_tree').jstree({
        core : {
          data : {
            url : function(node){
              if( node.id == '#' ){
                return "/api/?object=place&tree&filter=place:0"
              }else{
                let regexp = new RegExp( /^(.*)_([0-9]*)$/ );
                return "/api/?object=place&tree&filter=place:" + Number( node.id.replace( regexp , function(_,other,id){
                                                                  return id;
                                                                }) )
              }
            },
          },
          'themes': {
                        'name': 'proton',
                        'responsive': true,
          },
          'check_callback' : function(operation, node, node_parent, node_position, more){
              return true;
          },
        },
        //"checkbox": {
        //    "three_state": false
        //},
        plugins : ["ui","checkbox"]
      })
      
      .bind('load_node.jstree' , function( e , core ){
        
        var core = core
        
        mails.fetch({
          data : {
            filter : "id:" + $('[name="Id"]').val(),
            place_open : 1
          },
          success : function( data , res){
            opened = res
            core.instance.open_node( opened.reduce(function( res , item ){
              res.push( item.id )
              return res;
            },[]) )
          }
        })
        
        core.instance.open_node( opened.reduce(function( res , item ){
              res.push( item.id )
              return res;
            },[]) )
        
        core.instance.select_node( $('#selected_places').val().split(",") );
        
      })
      
      .bind('select_node.jstree' , function(){
        callback_step_4()
        $('.modal').trigger('callback_step_5')
      })
      
      .bind('deselect_node.jstree' , function(){
        callback_step_4()
        $('.modal').trigger('callback_step_5')
      })
      
      .bind('open_node.jstree' , function( _ , core ){
        core.instance.select_node( $('#selected_places').val().split(",") );
      })
      
      
    }
    
  }
  
  
  
  
  /*
    Првоерка валидности 4 шага
    @returns {Bool}
  */
  function is_valid_step_4(){
    return Boolean( typeof $('#container_tree').data('jstree') != 'undefined' &&
          $('#container_tree').jstree(true).get_selected().length )
  }
  
  
  
  /*
    Коллбек проверки валидности 4 шага
    @returns {Bool} false
  */
  function callback_step_4(){
    
    let valid = is_valid_step_4() &&
                is_valid_step_3() &&
                is_valid_step_2() &&
                is_valid_step_1();
    
    let f_1 = valid ? 'removeClass' : 'addClass'
    let f_2 = valid ? 'addClass' : 'removeClass'
    
    
    let i = 5;
        
    $('#step_' + i).find('.circle')[f_1]( DISABLE_I_CLASS )[f_2]( NOTACTIVE_I_CLASS )
    $('#step_' + i).find('.header')[
      valid ? 'removeClass' : 'addClass'
    ]( 'text-muted' )
    if( valid ){
      $('#step_5').find('select')[
        valid ? 'removeClass' : 'addClass'
      ]('hide')
      console.log(  );
    }
    
    return false;
  }
  
  
  
  
  /*
    Проверка валидности шага 5
    @returns {Bool}
  */
  function is_valid_step_5(){
    return $('#type_report').val() != 0 && 
                is_valid_step_4() &&
                is_valid_step_3() &&
                is_valid_step_2() &&
                is_valid_step_1();
  }
  
  
  
  
  
  
  /*
    Коллбек после выполнения шага 5
    @returns {Bool} false
  */
  function callback_step_5( tpl ){
    if( is_valid_step_5() ){
      if( $('.modal-footer').hasClass('hide') ){
        $('.modal-footer').removeClass('hide').hide().slideDown(300)
      }
    }else{
      $.when( $('.modal-footer').slideUp(300 ) ).then( function( footer ){
        $(footer).addClass('hide')
      })
    }
  }

  
  
  
  
  
  
  
  
  /*
    Общте действия при нажатиии на кнопку шага
    @return false
  */
  function click_circle_btn(){
    
    //Можно кликать только по тем элементам которые котрыты для редактирования
    //и сейчас неактивны
    if( $(this).hasClass( NOTACTIVE_I_CLASS ) ){
      
      //Нужно закрыть все активные элементы как сладер
      //и открыть этот тоже как слайдер
      
      $( this ).closest('.milestones')
              .find( 'i.' + ACTIVE_I_CLASS )
              .closest('li')
              .find('.toggle_body')
              .slideUp( 300 )
              
      $(this).closest('.milestones')
            .find('.circle.'+ACTIVE_I_CLASS)
            .removeClass( ACTIVE_I_CLASS )
            .addClass( NOTACTIVE_I_CLASS )
              
      $( this ).closest('li')
              .find('.toggle_body')
              .removeClass('hide')
              .hide()
              .slideDown( 300 )
              
      $( this ).removeClass( NOTACTIVE_I_CLASS ).addClass( ACTIVE_I_CLASS )
    }
    return false;
  }
  
  
  
  /*
    Общте действия при нажатиии на название шага
    @see click_circle_btn
    @return false
  */
  function click_header(){
    $( this ).closest('li').find('.circle').trigger('click')
    return false;
  }
  
  
  
  
  
  /*
    Выбор отчета на каждый день
    @param {Event}
    @param {DOM}
    @returns {Bool} false
  */
  function every_day( e , tpl ){
    $( e.target ).closest('.toggle_body').find('a').each(function( i , a ){
      if( !$(a).hasClass('every_day') ){
        $(a).removeClass('active')
      }
    })
    $( e.target ).toggleClass('active')
    
    $( tpl ).trigger( 'callback_step_3' )
    
    return false;
  }
  
  
  
  
  /*
    Выбор дня недели
    @param {Event}
    @param {DOM}
    @returns {Bool} false
  */
  function custom_day( e , tpl ){
    $( e.target ).closest('.toggle_body').find('a').each(function( i , a ){
      if( !$(a).hasClass('custom_day') ){
        $(a).removeClass('active')
      }
    })
    //$( e.target ).addClass('active')
    
    if( $( e.target ).closest('.custom_day').hasClass('active') ){
      
      $('#datepicker').datepicker({
        format: "dd",
        maxViewMode: 0,
        todayBtn: "linked",
        language: "ru",
        orientation: "bottom right",
        multidate: true,
        multidateSeparator: ",",
        calendarWeeks: true,
        clearBtn : 'Закрыть',
        todayHighlight: true
      })
      .on('changeDate', function(e) {
        $( tpl ).trigger( 'callback_step_3' )
      });
      $('#datepicker').datepicker('show');
    }
    
    $( tpl ).trigger( 'callback_step_3' )
    
    return false;
  }
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  /*
    Выбор дня недели
    @param {Event}
    @param {DOM}
    @returns {Bool} false
  */
  function custom_week_day( e , tpl ){
    
    
    /*
      Получение дней которые уже были выделены
      @returns {Array}
    */
    var ArrayWeekDay = function(){
      return $('#custom_day_week_res').val().split(',').reduce(function( res , item ){
        res.push( Number( item ) )
        return res;
      },[])
    }
    
    
    $( e.target ).closest('.toggle_body').find('a').each(function( i , a ){
      if( !$(a).hasClass('custom_week_day') ){
        $(a).removeClass('active')
      }
    })
    $( e.target ).addClass('active')
    
    let classes_datepicker = [ ".custom_week_day" , ".custom_day" ]
    
    var this_datepicker_show = $('.datepicker').is(':visible')
    
    classes_datepicker.map(function( class_name ){
      $('.custom_week_day').off('changeDate')
      $( class_name ).datepicker('destroy')
    })
    
    if( $( e.target ).closest('.custom_week_day').hasClass('active') ){
      
      if( this_datepicker_show ){
        return false;
      }
      
      $('.custom_week_day').datepicker( $('.datepicker').is(':visible') ? 'hide' : 'show')
    
      $('.custom_week_day').datepicker({
        format: "dd",
        maxViewMode: 0,
        language: "ru",
        orientation: "bottom right",
        firstDayOfWeek: 1,
        multidateSeparator: ",",
        calendarWeeks: true,
        clearBtn : true,
        todayHighlight: true,
      });
      
      
      $('.custom_week_day').on('changeDate', function( e , d ) {
        
        let day = new Date( e.date ).getDay()
        
        var a = ArrayWeekDay()
        
        if( !isNaN( day ) && "".concat(day).trim() != ""  ){
          if( a.indexOf( day ) >= 0 ) {
            a.splice( a.indexOf( day ) , 1 )
          }else{
            a.push(day)
          }
        }
        
        $('#custom_day_week_res').val( a.join(',') )
        $('.custom_week_day').datepicker('setDaysOfWeekHighlighted' , a )
        
        $(tpl).trigger('callback_step_3')
        
        return false;
        
      })
      .on('show' , function( e ){
        $('.custom_week_day').datepicker('setDaysOfWeekHighlighted' , ArrayWeekDay() )
      })
      
      
      $('.custom_week_day').trigger('changeDate')
      
      
      $('.custom_week_day').datepicker('show')
    }
    
    $( tpl ).trigger( 'callback_step_3' )
    
    return false;
  }
  
  
  
  
  
  
  
  /*
    Сохранение на сервер данных
    @param {Event}
    @param {Number}
    @param {DOM}
  */
  function save_to_serv( e , id , tpl ){
    
    var mail = new mails.model({
      id: Number( id ),
      crontab : function(){
        
        let hours     = "",
            minutes   = "",
            days      = "",
            months    = "*",
            day_weeks = ""
        
        $('.clockpicker').clockpicker().find('input').val().replace(/^([0-9]*)\:([0-9]*)$/,function( _, _h, _m ){
          hours     = _h;
          minutes  = _m
        })
        
        var step_3_active = $('#step_3').find('a.active')
        
        if( !$(step_3_active).length ){
          return "invalid_step_3";
        }
        
        // Каждый день
        if( $( step_3_active ).hasClass('every_day') ){
          days = "*"
        // По дням недели
        }else if( $( step_3_active ).hasClass('custom_week_day') ){
          day_weeks = $('#custom_day_week_res').val()
        }else if( $( step_3_active ).hasClass('custom_day') ){
          days      = $('#my_hidden_input').val()
        }
        
        if( !days && day_weeks ){
          days = "*"
        }else if( days && !day_weeks ){
          day_weeks = "*"
        }
        
        if( !hours ){
          hours = "*"
        }
        if( !minutes ){
          minutes = "0"
        }
        
        if( !hours || !minutes || !days || !months || !day_weeks ){
          return "invalid_cron";
        }
        
        return [Number(minutes),Number(hours),days,months,day_weeks].join(" ");
        
      }(),
      listmail : $('#listing_emails').find('.valid_email').toArray().reduce(function( res , item ){
        res.push( $(item).text().trim() )
        return res;
      },[]).join("\n"),
      enabled : $('#Enabled').is(':checked'),
      name : $('#Name').val(),
      type : $('#type_report').val(),
      ids : typeof $('#container_tree').data('jstree') != "undefined"
            ? $('#container_tree').jstree(true).get_selected().reduce(function( res , item ){
                if( 
                    $('#container_tree').jstree(true).get_node( 
                      $('#container_tree').jstree(true).get_node( item ).parent 
                    ).state.selected 
                  ){
                    return res;
                  }
                res.push( Number(item.replace(/^place_tree_([0-9]*)_([0-9]*)$/,function(_,parent_id , id){
                  return id
                })) )
                return res
              },[])
            : []
    })
    
    if( mail.get('crontab') == "invalid_cron" ){
      $(tpl).modal('hide')
      alertDanger("Не верно задано расписание")
      return false;
    }
    
    mail.save().then(function( e,s ){
      if( s == 'success' ){
        $(tpl).modal('hide')
        alertSuccess()
        mails.trigger( !id ? 'add_model' : 'change_model' , mail ) 
      }
    })
    
  }
  
  
  
  
  
  
  
  
  
  
  /*
    Показ модального окна с уведомлением о удачном завершении сохранения
  */
  function alertSuccess(){
    $('<div/>').addClass('modal fade').attr({
      tabindex          : -1,
      role              : "dialog",
      'aria-labelledby' : 'myModalLabel',
      'aria-hidden'     : 'true'
    }).append(
      $('<div/>').addClass('modal-dialog').append(
        $('<div/>').addClass('modal-content').append(
          $('<div/>').addClass('modal-header').append(
            $('<span/>').addClass('pull-left').css({
              padding : "5px"
            }).text('Шаблон успещно сохранен')
          ).append(
            $('<button/>').attr({
              type : "button"
            }).addClass('btn btn-success pull-right btn-circle').append(
              $('<i/>').addClass('fa fa-check')
            )
          )
        )
      )
    ).modal('show').on('shown.bs.modal',function(){
      var self = this
      setTimeout(function f(){
        $( self ).modal('hide')
      }, 1000);
    }).on('hidden.bs.modal', function () {
      $(this).remove()
    })
  }
  
  
  
  /*
    Показ модального окна с уведомлением о удачном завершении сохранения
  */
  function alertDanger( msg ){
    $('<div/>').addClass('modal fade').attr({
      tabindex          : -1,
      role              : "dialog",
      'aria-labelledby' : 'myModalLabel',
      'aria-hidden'     : 'true'
    }).append(
      $('<div/>').addClass('modal-dialog').append(
        $('<div/>').addClass('modal-content').append(
          $('<div/>').addClass('modal-header').append(
            $('<span/>').addClass('pull-left').css({
              padding : "5px"
            }).text( msg )
          ).append(
            $('<button/>').attr({
              type : "button"
            }).addClass('btn btn-danger pull-right btn-circle').append(
              $('<i/>').addClass('fa fa-times')
            )
          )
        )
      )
    ).modal('show').on('shown.bs.modal',function(){
      var self = this
      setTimeout(function f(){
        $( self ).modal('hide')
      }, 2000);
    }).on('hidden.bs.modal', function () {
      $(this).remove()
    })
  }
  
  
  
  
  
  
  
    
  
  
  return Backbone.View.extend({
    
    
    initialize : function( id , callback ){
      
      var data = { data : [ id ] }
      
      var self = this
      
      var id = id
      
      $.get(
        '/tpl/?tpl=mail_edit',
        data,
        function( tpl ){
          
          var tpl = $(tpl)
          
          $(tpl).find('[type="submit"]').remove()
          
          self.$el = $('#mail_modal_edit')
          
          $(tpl).modal('show')
          .on('hidden.bs.modal', function () {
            $(tpl).remove()
          })
          .on('shown.bs.modal'                                , function(){ show_modal( id , tpl )              })
          .on('click'                   , '.saved_email'      , function(){ saved_email.apply( this , tpl )     })
          .on('click'                   , '.edit_email'       , function(){ edit_email.apply( this , tpl )      })
          .on('click'                   , '.drop_email'       , function( e ){  drop_email( e , tpl )           })
          .on('click'                   , '.every_day'        , function( e ){  every_day( e , tpl )            })
          .on('click'                   , '.custom_day'       , function( e ){  custom_day( e , tpl )           })
          .on('click'                   , '.custom_week_day'  , function( e ){  custom_week_day( e , tpl )      })
          .on('click'                   , '.circle'           , function( e ){  click_circle_btn.apply( this )  })
          .on('click'                   , '.header'           , function( e ){  click_header.apply( this )      })
          .on('click'                   , '#append_email'     , append_mail                                     )
          .on('click'                   , '.every_time'       , function( e ){  every_time.apply( this , tpl )  })
          .on('click'                   , '.custom_time'      , function( e , t ){
                                                                  custom_time( e , tpl || t )
                                                                })
          .on('change'                  , '#type_report'      , function(){ callback_step_5.apply( this , tpl ) })
          
          .on('is_empty_listmail'                             , is_empty_listmail                               )
          
          // Функции которые вызываются после того как выполнен шаг
          .on('callback_step_1'                               , callback_step_1                                 )
          .on('callback_step_2'                               , callback_step_2                                 )
          .on('callback_step_3'                               , callback_step_3                                 )
          .on('callback_step_4'                               , callback_step_4                                 )
          .on('callback_step_5'                               , callback_step_5                                 )
          .on('click'                   , '#save_mail'        , function( e ){ save_to_serv( e , id , tpl )     })
          
          .on('keyup' , '#Name' , function(){
            $(tpl).trigger('callback_step_1')
          })
          
          if( callback ) callback()
          
        }
      )
      
    },
  
    el : '#mail_modal_edit',
  
  
  
  });
  
  
});