define([
  "backbone",
  "meterdatahours",
  "meterparameters",
  "highcharts",
  "exporting"
],function(
  Backbone,
  meterdatahours,
  meterparameters,
  Highcharts
){
  
  Array.prototype.min = function() {
    return Math.min.apply(null, this);
  };
  
  
  var Chart = Highcharts.chart( 'container_meter_graph' , {
      colors: ['#7cb57c', '#a8b4bd', '#2677b5'],
      chart: {
          type: 'areaspline'
      },
      title: {
          text: ''
      },
      subtitle: {
          text: ''
      },
      tooltip: {
          pointFormat: '{series.name} produced <b>{point.y:,.0f}</b><br/>warheads in {point.x}'
      },
      plotOptions: {

          areaspline: {
            stacking: 'normal',
            lineColor: '#666666',
            lineWidth: 1,
            pointStart: 2010,
            pointInterval: 0.3, // hourly data
            marker: {
              enabled: true,
              symbol: 'circle',
              radius: 3,
              states: {
                hover: {
                  enabled: true
                }
              }
            }
          }
      },
  });
  

  return Backbone.View.extend({
    
    
    setParameter : function( pid ){
      
      var self = this
      
      
      
      var regexpArchive = new RegExp(/\graphs\/(\d{4})(\d{2})(\d{2})-(\d{4})(\d{2})(\d{2})\//);
      
      
      if( window.location.pathname.match( regexpArchive ) ){
        
        var serries = meterdatahours.clone()
        
        window.location.pathname.replace( regexpArchive ,function(_,start_y,start_m,start_d,stop_y,stop_m,stop_d){
          
          var startDate = new Date( start_y , start_m , start_d );
          var stopDate  = new Date( stop_y , stop_m , stop_d );
          var diffDays  = Math.abs( startDate - stopDate ) / 1000 / 60 / 60 / 24;
          
          startDate.setDate(startDate.getDate() -1)
          
            
            
            meterparameters.fetch({
              data : {
                filter : "virtmeter:" + self.get_id + ";parameter:" + pid
              },
              success:function(data){
                
                
                var mp = data.last()
                
                
                for( i = 0 ; i <= diffDays ; i++ ){
                  startDate.setDate(startDate.getDate() + 1)
                  
                  var parameters = [
                    "meter:" + mp.get('meter'),
                    "parameter:" + pid,
                    "year:" + startDate.getFullYear(),
                    "month:" + startDate.getMonth(),
                    "day:" + startDate.getDate(),
                  ]
                  
                  serries.fetch({
                    reset:false,
                    remove : false,
                    add:true,
                    silent: true,
                    data : {
                      filter : parameters.join(";")
                    },
                    success : function( data ){
                      self.collection = data
                      
                      var data_first = self.collection.where({ parameter : Number(pid) })
                      while(Chart.series.length > 0){
                        Chart.series[0].remove(true);
                      }
                      
                      Chart.addSeries({
                          name: $('#parameter_'+pid).text(),
                          data: data_first.sort(function(a,b){
                        
                            var a_Date = new Date( a.get('year') , a.get('month') , a.get('day') );
                            var b_Date = new Date( b.get('year') , b.get('month') , b.get('day') );
                            
                            if( a_Date.getTime() == b_Date.getTime() ) return 0;
                            return a_Date.getTime() < b_Date.getTime() ? -1 : 1;
                          }).reduce( function( res , item){
                            res.push( item.get('valueconv') )
                            return res;
                          }, [])
                      }, false);
                      
                      
                      Chart.yAxis[0].update({
                          min: data_first.reduce(function(res , item){
                            res.push(item.get('valueconv'))
                            return res;
                          } , []).min()
                      }); 
                      
                      Chart.redraw();
                      
                    }
                  })
                }
              }
            })
          })
            
      }else{
        meterparameters.fetch({
          data : {
            filter : "virtmeter:" + this.get_id + ";parameter:" + pid
          },
          success:function(data){
            
            
            var mp = data.last()
            
            
            
            meterdatahours.fetch({
              data : {
                filter : "meter:" + mp.get('meter') + ";parameter:" + pid
              },
              success : function( data ){
                self.collection = data
                                              
                
                var data_first = self.collection.where({ parameter : Number(pid) })
                while(Chart.series.length > 0){
                  Chart.series[0].remove(true);
                }
                
                Chart.addSeries({
                    name: $('#parameter_'+pid).text(),
                    data: data_first.sort(function(a,b){
                      if( a.get('hour') == b.get('hour') ) return 0;
                      return a.get('hour') < b.get('hour') ? -1 : 1;
                    }).reduce( function( res , item){
                      res.push( item.get('valueconv') )
                      return res;
                    }, [])
                }, false);
                
                
                Chart.yAxis[0].update({
                    min: data_first.reduce(function(res , item){
                      res.push(item.get('valueconv'))
                      return res;
                    } , []).min()
                }); 
                
                Chart.redraw();
                
              }
            })
            
            
            
            
          }
        })
      }
      
      
    },
  
  
    initialize : function(id , year , month , day){
      
      $('.change_parameter').removeClass('active')
      
      this.get_id = id
      
      var first_patameter_el = $('#block_parameters').find('a')
                                        .first()
      $(first_patameter_el).addClass('active')
      
      if( !$(first_patameter_el).length ) return false;
      
      var pid = $(first_patameter_el).attr('id').replace('parameter_','')
      
      this.setParameter( pid )
      
    },
  
  
  
    render : function(){
  
    },
  
  
    events : {
      'click .change_parameter' : function( e ){
        
        $('.change_parameter').removeClass('active')
        
        var first_patameter_el = $(e.target).closest('.change_parameter')
        
        $(first_patameter_el).addClass('active')
        
        var pid = $(first_patameter_el).attr('id').replace('parameter_','')
        
        this.setParameter( pid )
      }
    },
  
    el : '#main_block_graphs',
  
  
  
  });


});