/* global Backbone,  _ */
if (typeof error_invalid_place === 'undefined') {
    var error_invalid_place = "Неверно выбран объект"
}
if (typeof error_no_places === 'undefined') {
    var error_no_places = "Не выбраны объекты"
}
if (typeof report_by_places === 'undefined') {
    var report_by_places = "Отчет по местам"
}
if (typeof report_by_converters === 'undefined') {
    var report_by_converters = "Отчет по приборам учета"
}

var TYPE_OBJECT = Backbone.Model.extend({
  defaults : {
    type        : "place",
    ids  : ""
  },
  
  
  validate: function( ) {
    
    var attrs = this.toJSON();
    
		if ( $('#type_object').find('option').toArray().reduce(function(res,item){
        res.push($(item).val());
        return res;
      },[]).indexOf( attrs.type ) === -1 ) {
			return error_invalid_place;
		}
    if ( attrs.ids === this.defaults.ids ) {
			return error_no_places;
		}
 
	}
  
});

var type_object = new TYPE_OBJECT();


var TYPE_OBJECT_VIEW = Backbone.View.extend({
  
  el : "#type_object",
  
  model : type_object,
  
  events : {
    
    'change' : function( e ){
      this.model.set({ type : $(e.target).val() });
      return false;
    }
    
  }
  
});



var TYPE_NAME_OBJECT = Backbone.View.extend({
  
  el : '#name_object',
  
  model : type_object,
  
  initialize : function(){
    this.listenTo( this.model , 'change' ,this.render );
    this.render();
  },
  
  render : function(){
    var self = this;
    $( this.$el ).text(function(){
      switch( self.model.get('type') ){
        case "place" : return report_by_places;
        case "meter": return report_by_converters;
      }
    });
  }
  
});




var TREE_VIEW = Backbone.View.extend({
  
  el : '#tree_map',
  
  model : type_object,
  
  initialize : function(){
    this.listenTo( this.model , 'change' , this.render );
    this.render();
  },
  
  render : function(){
    
    
    if( $( this.$el ).data("jstree") ){
      $( this.$el ).jstree('destroy');
    }
    
    var data_type = {};
    
    switch( type_object.get('type') ){
      case 'place' :
        data_type = {
          'url' : function (node) {
            if( node.id === '#' ){
              return '/api/place/?jstree&all';
            }else{
              return '/api/place/?jstree&all&place=' + node.id;
            }
          }
        };
        break;
      case 'meter' : 
        data_type = {
          'url' : function (node) {
            if( node.id === '#' ){
              return '/api/?object=converter&tree=true';
            }else{
              return '/api/?object=meter&tree=true&filter=converter:' + node.id.replace( 'converter_tree_','');
            }
          }
        };
        break;
    }
    
    
    $( this.$el ).jstree({
      "core" : {
        "check_callback" : function () {
          return true;
        },
        'themes': {
            'name': 'proton',
            'responsive': true
        },
        'data' : data_type
      },
      "checkbox": {
          "three_state": false
      },
      "plugins" : [
        "json","types","checkbox"
      ]

    }).bind('loaded.jstree' , function( i,core ){
      core.instance.uncheck_all();
    })
    .bind('load_node.jstree' , function( instance , core ){
      core.node.children.map(function( node_id ){
        if( !node_id.match( /meter_tree_([0-9]*)/ ) ){
          return false;
        }
        $( "#" + node_id ).find('>a> .jstree-icon.jstree-checkbox').remove();
      });
    })
    .bind('open_node.jstree' , function( instance , core ){
      core.node.children.map(function( node_id ){
        if( !node_id.match( /meter_tree_([0-9]*)/ ) ){
          return false;
        }
        $( "#" + node_id ).find('>a> .jstree-icon.jstree-checkbox').remove();
      });
    })
    .bind('select_node.jstree' , function( instance , core ){
      replaceIds();
    })
    .bind('deselect_node.jstree' , function( instance , core ){
      replaceIds();
    })
    ;
    
    
    
  }
  
});


function replaceIds(){
  type_object.set({ ids : $('#tree_map').jstree('get_selected').reduce(function( res , item ){
      res.push( item.replace('converter_tree_','') );
      return res;
  },[]).join(',') 
  },{silent:true});
}



var HIDDEN_FORM = Backbone.View.extend({
  
  el : '#hidden_form',
  
  template : '#hidden_form_tpl',
  
  model : type_object,
  
  initialize : function(){
    this.listenTo( this.model , 'change' , this.render );
    this.render();
  },
  
  
  render : function(){
    var tpl_c = _.template( $( this.template ).html() );
    $( this.$el ).empty().append(
              tpl_c({
                res : this.model.toJSON()
              })
            );
  }
  
});

var hidden;


$(document).ready(function(){
  
  new TYPE_OBJECT_VIEW();
  new TYPE_NAME_OBJECT();
  new TREE_VIEW();
  hidden = new HIDDEN_FORM();
  
}).on('click','#send' , function(){
  
  
  if( !type_object.isValid() ){
    return false;
  }
  
  $('#init_form').remove();
  
  hidden.render();
  
  var iframe = $('<iframe>',{
      width:1,
      height:1,
      frameborder:0,
      css:{
          display:'none'
      }
  }).attr('id' , 'init_form').appendTo('body');
  
  
  var formHTML = $('#hidden_form').html();
  
  var body = (typeof iframe.prop('contentDocument') !== 'undefined') ?
                  iframe.prop('contentDocument').body :
                  iframe.prop('document').body;
          
  body = $(body);

  body.html(formHTML);
  
  var form = body.find('form');
  
  
  form.submit();
  
  return false;
  
});