/* global Backbone, _, PLACE_CHILDREN_LIST_PAGINATION, PLACE_MODAL_INFO, PLACE_CHILDREN_LIST, PLACE_MODAL_TREE, place, PLACE, PLACE_MODAL_METERS, self */


var PLACE_H1 = Backbone.View.extend({
  
  el : '.container_h1_place',
  
  model : place,
  
  initialize : function(){
    this.listenTo( this.model , 'change' , this.render );
  },
  
  render : function(){
    $( this.$el ).empty().append( this.model.escape('title') );
  }
  
});


/**
 * Вид для работы со страницей места
 *
 * @type Backbone.View
 */
var PLACE_INDEX = Backbone.View.extend({

  el : '#place_children_table',


  initialize : function(){

    var self = this;


    /**
     * Отслеживаем добавление мест
     */
    this.listenTo( this.collection , 'add remove' , function(){
      if( typeof PLACE_CHILDREN_LIST !== "undefined" ){
        new PLACE_CHILDREN_LIST( self.collection );
      }else{
        console.log("Не включен модуль PLACE_CHILDREN_LIST");
      }
      if( typeof PLACE_CHILDREN_LIST_PAGINATION !== "undefined" ){
        new PLACE_CHILDREN_LIST_PAGINATION( self.collection );
      }else{
        console.log("Не включен модуль PLACE_CHILDREN_LIST_PAGINATION");
      }
    });
  },



  model : place,



  /**
   *
   * @type {Backbone.Collection} Коллекция мест учета
   */
  collection : new PLACES(),



  events : {


    /**
     * Вывод модального окна для добавления дочернег места в место
     *
     * @param {Object} e event
     * @returns {Boolean}
     */
    'click .add_child' : function( e ){
      
      var place_parent = this.model ? this.model : new PLACE();
      
      var place = new PLACE({ place : place_parent.get('id') });
      this.collection.add(place , {silent : true});
      if( typeof PLACE_MODAL_INFO === 'undefined' ){
        return true;
      }
      if( typeof PLACE === 'undefined' ){
        return true;
      }
      
      new PLACE_MODAL_INFO( place , place_parent );
      return false;
    },



    /**
     * Удаление одного места
     *
     * @param {Event} e
     * @returns {Boolean}
     */
    'click .children_delete' : function( e ){
      var tr = $( e.target ).closest('tr');

      if( !$( tr ).data('droped') ){
        swal({
          title: "Нельзя удалить.",
          text: "Удалите вложенные элементы и связи с этим местом.",
          type: "warning",
          confirmButtonClass : "btn-default"
        });
        return false;
      }

      var id = $( tr ).attr('id').replace('children_place_','');

      var place = new PLACE({ id : Number( id ) , title : $( tr ).find('.link_to_place').find('span').text() });

      new PLACE_MODAL_DELETE( place , this.collection );
      return false;
    },


    /**
     *
     * @param {Event} e
     * @returns {Boolean}
     */
    'click .children_edit' : function( e ){

      var tr = $( e.target ).closest('tr');
      var id = $( tr ).attr('id').replace('children_place_','');

      // проверка на существование модального окна

      var place = new PLACE({ id : id });
      this.collection.add(place , {silent : true});
      place.fetch({
        success : function(){
          new PLACE_MODAL_INFO( place , new PLACE() );
        }
      });

      return false;
    }

  }

});







/**
 * Контроллер отслеживающий действия на блоке действий
 *
 * @type Backbone.View
 */
var PLACE_ACTION = Backbone.View.extend({

  el : '#block_action_place',

  model : place,

  events : {


    /**
     * Показ мест в виде дерева
     *
     * @param {Event} e
     * @returns {false}
     */
    'click #view_to_tree' : function( e ){

      if( typeof PLACE_MODAL_TREE === 'undefined' ){
        console.log("Не подключен модуль PLACE_MODAL_TREE");
        return false;
      }
      
      new PLACE_MODAL_TREE( this.model );
      
      return false;
    },
    
    
    
    
    /**
     * Редактирование текущего места
     * 
     * @returns {Boolean}
     */
    'click #controller_edit_place' : function(){
      if( typeof PLACE_MODAL_INFO === 'undefined' ){
        return true;
      }
      if( typeof PLACE === 'undefined' ){
        return true;
      }
      
      var self = this;
      
      self.model.fetch({
        success : function(){
          new PLACE_MODAL_INFO( self.model , new PLACE() );
        }
      });
      return false;
    },
    
    
    
    
    
    /**
     * Редактирование приборов учета
     * 
     * @returns {Boolean}
     */
    "click #edit_meters" : function(){
      
      if( typeof PLACE_MODAL_METERS === 'undefined' ){
        return false;
      }

      new PLACE_MODAL_METERS( self.model );
      return false;
    }


  }


});





/**
 * Вид овечающий за актуальную таблицу с информацией о месте
 * 
 * @type Backbone.View
 */
var PLACE_INFO = Backbone.View.extend({
  
  //place_one_info
  
  el : '#place_item_info',
  
  model : place,
  
  initialize : function(){
    this.listenTo( this.model , 'change' , this.render );
  },
  
  render : function(){
    
    var self = this;
    
    $.get(
            '/tpl/?tpl=place_one_info',
            {data : [ this.model.get('id') ] },
            function( d ){
              $( "#place_item_info" ).closest('.panel').replaceWith( $(d) );
            }
    );
  }
  
});







/**
 * Вид овечающий за актуальную таблицу с информацией о месте
 * 
 * @type Backbone.View
 */
var PLACE_BREADCRUMB = Backbone.View.extend({
  
  el : '.breadcrumb .active',
  
  model : place,
  
  initialize : function(){
    this.listenTo( this.model , 'change' , this.render );
  },
  
  render : function(){
    $( this.$el ).empty().append( this.model.get('title') );
  }
  
});







$(document).ready(function(){
  new PLACE_H1();
  new PLACE_BREADCRUMB();
  new PLACE_INFO();
  new PLACE_INDEX();
  new PLACE_ACTION();
  return false;
});
