/* global Backbone, _, e, self, MODAL_SUCCESS */


/**
 * Модальное окно редактирования места
 * 
 * @type Backbone.View
 */
var PLACE_MODAL_INFO = Backbone.View.extend({
  
  template : '#place_modal_children_tpl',
  
  el : '#place_modal_children',
  
  
  
  /**
   * Указывает на то что мы добавялем или меняем место
   * 
   * @type {Boolean}
   */
  is_new : false,
  
  
  /**
   * Создание модального окна для редактирования или добавления мест
   * 
   * @param {PLACE} place То место которое хотим добавить или обновить
   * @param {PLACE} parent_place Место в которое быдем записывать
   *                Этот параметр необходим для того чтобы привязвыть место
   *                при создании
   * @returns {Boolean}
   * 
   */
  initialize : function( place , parent_place ){
    
    this.model = place;
    
    this.is_new = place.get('id') === 0;
    
    this.parent_place = parent_place;
    
    var self = this;
    
    var tpl_c = _.template( $( this.template ).html() );
    var modal_tpl = tpl_c({
                      place         : this.model,
                      place_parent  : this.parent_place
                    });
    
    this.$el = $( modal_tpl );
    
    $(this.$el).on('hidden.bs.modal', function (e) {
      $('#place_modal_children').remove();
      return true;
    }).on('hide.bs.modal', function ( e ) {
      return self.valid;
    }).on('shown.bs.modal' , function(){
      $( '[name="Title"]').focus();
    });
    
    this.render();
    
  },
  
  
  /**
   * Проверка валидации формы места
   * 
   * @returns {Boolean}
   */
  validate_form : function(){
    var valid = this.model.validate();

    switch( valid ){
      case 'empty_title' :
        $('#place_form_title').addClass('has-error');
        if( !$('#place_form_title').find('.form_error').length ){
          $('#place_form_title').find('label').after(
                $('<p/>').text('Вы не ввели название места').addClass('text-danger form_error')
          );
        }
        break;
      default :
        $('#place_form_title').removeClass('has-error').find('.form_error').remove();
        break;
    }
    return true;
  },
  
  
  
  /**
   * Блокировка формы
   * 
   * @returns {Boolean}
   */
  preloader : function(){
    $('.preloader').slideDown(300);
    $('.main_buttons').slideUp(300);
    $('.form-control').attr('disabled' , true);
    return true;
  },
  
  
  
  /**
   * Разблокировка формы
   * 
   * @returns {Boolean}
   */
  stoploader : function(){
    $('.preloader').slideUp(300);
    $('.main_buttons').slideDown(300);
    $('.form-control').attr('disabled' , false);
    $(this.$el).modal('hide');
    if( typeof MODAL_SUCCESS !== 'undefined' ){
      var action = this.is_new ? "Добавлено" : "Изменено";
      new MODAL_SUCCESS( action );
    }else{
      console.log("Не подключен модуль MODAL_SUCCESS");
    }
    return true;
  },
  
  
  
  
  /**
   * Вывод ошибки добавления
   * 
   * @returns {Boolean}
   */
  error_block : function(){
    if( !$('.modal-body').find('.alert-danger').length ){
      $('.modal-body').append(
              $('<div/>').addClass('alert alert-danger').text('Ошибка при добавлении. попробуйте позже.')
              );
    }
    return true;
  },
  
  
  
  
  /**
    * Изменение поля названия места
    * 
    * @param {Event} e
    * @returns {Boolean}
    */
  change_name : function( e ){
    this.model.set({ title : $(e.target).val() } , {silent : true});
    this.validate_form();
    return false;
  },
  
  
  
  /**
    * Изменение поля "Лицевой счет"
    * 
    * @param {Event} e
    * @returns {Boolean}
    */
  change_bank_book : function( e ){
    this.model.set({ bankbook : $(e.target).val() } , {silent : true});
    return false;
  },
  
  
  
  
  /**
   * События на окне
   */
  events : {
    
    
    
    /**
     * Изменение поля названия места
     * 
     * @param {Event} e
     * @returns {Boolean}
     */
    'keyup [name="Title"]' : function( e ){ return this.change_name( e ); },
    'paste [name="Title"]' : function( e ){ return this.change_name( e ); },
    'blur [name="Title"]'  : function( e ){ return this.change_name( e ); },
    
    
    
    /**
     * Изменение поля "Лицевой счет"
     * 
     * @param {Event} e
     * @returns {Boolean}
     */
    'keyup [name="BankBook"]' : function( e ){ return this.change_bank_book( e ); },
    'paste [name="BankBook"]' : function( e ){ return this.change_bank_book( e ); },
    'blur [name="BankBook"]'  : function( e ){ return this.change_bank_book( e ); },
    
    
    
    /**
     * Нажатие на кнопку добавления места
     * 
     * @returns {Boolean}
     */
    'click #add_children' : function(){
      
      var self = this;
      
      var valid = this.model.validate();
      
      if( typeof valid === 'undefined' ){
        
        this.preloader();
        
        this.model.save( null , {
          success : function( model ){
            self.stoploader();
            model.trigger('change');
            if( model.collection ){
              model.collection.trigger('add');
            }
          },
          error: function (model, response , e) {
            self.error_block();
            self.stoploader();
          }
        });
      }else{
        this.validate_form();
      }
      return true;
    },
    
    
    
    
    
    /**
     * Смена типа места
     * 
     * @param {Event} e
     * @returns {Boolean}
     */
    'change #place_change_type' : function( e ){
      this.model.set({ type : $(e.target).val() }, {silent : true});
      $('#place_bankbook')[ Number( $(e.target).val() ) === 2 ? 'show' : 'hide' ]();
      return false;
    }
    
  },
  
  
  
  
  
  
  
  /**
   * Отрисовка модального окна редактирования места
   * 
   * @returns {PLACE_MODAL_INFO}
   */
  render : function(){
    $( this.$el ).modal('show');
    return this;
  }
  
});
