/* global Backbone, PLACE, _ */

/**
 * Показ модального окна для дерева мест
 *
 * @type Backbone.View
 */
var PLACE_MODAL_TREE = Backbone.View.extend({

  template : '#place_modal_tree_tpl',
  
  collection : new PLACES(),


  /**
   *
   * @param {PLACE} place
   * @returns {Backbone.View}
   */
  initialize : function( place ){

    if( !place || !( place instanceof PLACE ) ){
      return this;
    }

    this.model = place;

    var tpl_c = _.template( $( this.template ).html() );
    var modal_tpl = tpl_c({
                      place         : this.model
                    });

    this.$el = $( modal_tpl );

    $(this.$el).on('hidden.bs.modal', function (e) {
      $('#place_modal_tree').remove();
      return true;
    });
    
    this.render();
    return this;
  },


  /**
   *
   * @returns {DOM}
   */
  render : function(){
    
    var self = this;

    $( this.$el ).modal('show').find('.modal-body').jstree({
      "core" : {
        "check_callback" : function () {
          return true;
        },
        'themes': {
            'name': 'proton',
            'responsive': true
        },
        'data' : {
          'url' : function (node) {
            if( node.id === '#' ){
              return '/api/place/?jstree&all';
            }else{
              return '/api/place/?jstree&all&place=' + node.id;
            }
          }
        }
      },
      "checkbox": {
          "three_state": false
      },
      "plugins" : [
        "json","types"
      ]

    }).bind('select_node.jstree' , function( e,core ){
      if( core.node.original.a_attr.href === window.location.pathname ){
        return false;
      }
      window.location.pathname = core.node.original.a_attr.href;
      return false;
    }).bind('load_node.jstree' , function( e , core ){
      
      var core = core;
      
      self.collection.fetch({
        data : {
          id : self.model.get('id'),
          parents : true
        },
        success : function( collection ){
          collection.add( self.model )
          collection.toArray().map(function( place ){
            core.instance.open_node(place.get('id'));
          })
        }
      });
      
      return false;
    }).bind( 'open_node.jstree' , function( e,core ){
      
      if( core.node.id == self.model.get('id') ){
        core.instance.select_node( core.node.id );
        core.instance.deselect_node( "0" );
      }
      
      core.node.children.map(function( id ){
        if( id == self.model.get('id') ){
          core.instance.select_node( id );
          core.instance.deselect_node( "0" );
        }
      });
      
    });

    return this.$el;
  }

});
